# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Signal,
    Qt,
    QSize,
)
from PySide6.QtGui import (
    QMouseEvent,
    QKeyEvent,
    QFont,
)
from PySide6.QtWidgets import (
    QHBoxLayout,
    QSizePolicy,
    QFrame,
    QLabel,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////

# ////// TYPE HINTS IMPROVEMENTS FOR PYSIDE6 6.9.1
from typing import Optional

# UTILITY FUNCTIONS
# ///////////////////////////////////////////////////////////////

# CLASS
# ///////////////////////////////////////////////////////////////


class ClickableTagLabel(QFrame):
    """
    Tag-like clickable label with toggleable state.

    Features:
        - Clickable tag with enabled/disabled state
        - Emits signals on click and state change
        - Customizable text, font, min width/height
        - Customizable status color (traditional name or hex)
        - QSS-friendly (type/class/status properties)
        - Automatic minimum size calculation
        - Keyboard focus and accessibility

    Parameters
    ----------
    name : str, optional
        Text to display in the tag (default: "").
    enabled : bool, optional
        Initial state (default: False).
    status_color : str, optional
        Color when selected (default: "#0078d4").
    min_width : int, optional
        Minimum width (default: None, auto-calculated).
    min_height : int, optional
        Minimum height (default: None, auto-calculated).
    parent : QWidget, optional
        Parent widget (default: None).
    *args, **kwargs :
        Additional arguments passed to QFrame.

    Properties
    ----------
    name : str
        Get or set the tag text.
    enabled : bool
        Get or set the enabled state.
    status_color : str
        Get or set the status color.
    min_width : int
        Get or set the minimum width.
    min_height : int
        Get or set the minimum height.

    Signals
    -------
    clicked()
        Emitted when the tag is clicked.
    toggle_keyword(str)
        Emitted with the tag name when toggled.
    stateChanged(bool)
        Emitted when the enabled state changes.
    """

    clicked = Signal()
    toggle_keyword = Signal(str)
    stateChanged = Signal(bool)

    # INITIALIZATION
    # ///////////////////////////////////////////////////////////////

    def __init__(
        self,
        name: str = "",
        enabled: bool = False,
        status_color: str = "#0078d4",
        min_width: Optional[int] = None,
        min_height: Optional[int] = None,
        parent=None,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(parent, *args, **kwargs)

        # ////// SET PROPERTY TYPE FOR QSS
        self.setProperty("type", "ClickableTagLabel")

        # ////// INITIALIZE PROPERTIES
        self._name: str = name
        self._enabled: bool = enabled
        self._status_color: str = status_color
        self._min_width: Optional[int] = min_width
        self._min_height: Optional[int] = min_height

        # ////// SETUP UI
        self._setup_ui()
        self._update_display()

    def _setup_ui(self) -> None:
        """Setup the user interface components."""
        # ////// SETUP FRAME
        self.setFrameShape(QFrame.NoFrame)
        self.setFrameShadow(QFrame.Raised)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        self.setFocusPolicy(Qt.StrongFocus)
        self.setCursor(Qt.PointingHandCursor)
        self.setContentsMargins(4, 0, 4, 0)
        self.setFixedHeight(20)

        # ////// CREATE LAYOUT
        self._layout = QHBoxLayout(self)
        self._layout.setObjectName("status_HLayout")
        self._layout.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(12)

        # ////// CREATE LABEL
        self._label = QLabel()
        self._label.setObjectName("tag")
        self._label.setFont(QFont("Segoe UI", 8))
        self._label.setLineWidth(0)
        self._label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        self._label.setAlignment(Qt.AlignmentFlag.AlignHCenter)

        # ////// ADD LABEL TO LAYOUT
        self._layout.addWidget(self._label, 0, Qt.AlignmentFlag.AlignTop)

        # ////// SET MINIMUM SIZES
        if self._min_width:
            self.setMinimumWidth(self._min_width)
        if self._min_height:
            self.setMinimumHeight(self._min_height)

    def _update_display(self) -> None:
        """Update the display based on current state."""
        # ////// UPDATE LABEL TEXT
        self._label.setText(self._name)
        self.setObjectName(self._name)

        # ////// UPDATE QSS PROPERTIES
        if self._enabled:
            self.setProperty("status", "selected")
            self._label.setStyleSheet(
                f"color: {self._status_color}; background-color: transparent; border: none;"
            )
        else:
            self.setProperty("status", "unselected")
            self._label.setStyleSheet(
                "color: rgb(86, 86, 86); background-color: transparent; border: none;"
            )

        # ////// REFRESH STYLE
        self.refresh_style()
        self.adjustSize()

    # PROPERTIES
    # ///////////////////////////////////////////////////////////////

    @property
    def name(self) -> str:
        """Get the tag text."""
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        """Set the tag text."""
        self._name = str(value)
        self._update_display()
        self.updateGeometry()

    @property
    def enabled(self) -> bool:
        """Get the enabled state."""
        return self._enabled

    @enabled.setter
    def enabled(self, value: bool) -> None:
        """Set the enabled state."""
        if value != self._enabled:
            self._enabled = bool(value)
            self._update_display()
            self.stateChanged.emit(self._enabled)

    @property
    def status_color(self) -> str:
        """Get the status color."""
        return self._status_color

    @status_color.setter
    def status_color(self, value: str) -> None:
        """Set the status color."""
        self._status_color = str(value)
        if self._enabled:
            self._label.setStyleSheet(
                f"color: {value}; background-color: transparent; border: none;"
            )
            self.refresh_style()

    @property
    def min_width(self) -> Optional[int]:
        """Get the minimum width."""
        return self._min_width

    @min_width.setter
    def min_width(self, value: Optional[int]) -> None:
        """Set the minimum width."""
        self._min_width = value
        if value:
            self.setMinimumWidth(value)
        self.updateGeometry()

    @property
    def min_height(self) -> Optional[int]:
        """Get the minimum height."""
        return self._min_height

    @min_height.setter
    def min_height(self, value: Optional[int]) -> None:
        """Set the minimum height."""
        self._min_height = value
        if value:
            self.setMinimumHeight(value)
        self.updateGeometry()

    # EVENT FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def mousePressEvent(self, event: QMouseEvent) -> None:
        """Handle mouse press events."""
        if event.button() == Qt.MouseButton.LeftButton:
            self.enabled = not self.enabled
            self.clicked.emit()
            self.toggle_keyword.emit(self._name)
        super().mousePressEvent(event)

    def keyPressEvent(self, event: QKeyEvent) -> None:
        """Handle key press events."""
        if event.key() in [Qt.Key_Space, Qt.Key_Return, Qt.Key_Enter]:
            self.enabled = not self.enabled
            self.clicked.emit()
            self.toggle_keyword.emit(self._name)
        else:
            super().keyPressEvent(event)

    # OVERRIDE FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def sizeHint(self) -> QSize:
        """Return the recommended size for the widget."""
        return QSize(80, 24)

    def minimumSizeHint(self) -> QSize:
        """Return the minimum size for the widget."""
        # ////// CALCULATE MINIMUM SIZE
        font_metrics = self._label.fontMetrics()
        text_width = font_metrics.horizontalAdvance(self._name)
        min_width = self._min_width if self._min_width is not None else text_width + 16
        min_height = (
            self._min_height
            if self._min_height is not None
            else max(font_metrics.height() + 8, 20)
        )

        return QSize(min_width, min_height)

    # STYLE FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def refresh_style(self) -> None:
        """Refresh the widget style."""
        self.style().unpolish(self)
        self.style().polish(self)
        self.update()



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
Tests unitaires pour le widget AutoCompleteInput.
"""

import pytest
from unittest.mock import MagicMock
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QApplication, QCompleter

from ezqt_widgets.input.auto_complete_input import AutoCompleteInput


pytestmark = pytest.mark.unit


class TestAutoCompleteInput:
    """Tests pour la classe AutoCompleteInput."""

    def test_auto_complete_input_creation_default(self, qt_widget_cleanup):
        """Test de création avec paramètres par défaut."""
        input_widget = AutoCompleteInput()

        assert input_widget is not None
        assert isinstance(input_widget, AutoCompleteInput)
        assert input_widget.suggestions == []
        assert input_widget.case_sensitive is False
        assert input_widget.filter_mode == Qt.MatchContains
        assert input_widget.completion_mode == QCompleter.PopupCompletion

    def test_auto_complete_input_creation_with_parameters(self, qt_widget_cleanup):
        """Test de création avec paramètres personnalisés."""
        suggestions = ["test1", "test2", "test3"]
        input_widget = AutoCompleteInput(
            suggestions=suggestions,
            case_sensitive=True,
            filter_mode=Qt.MatchStartsWith,
            completion_mode=QCompleter.InlineCompletion,
        )

        assert input_widget.suggestions == suggestions
        assert input_widget.case_sensitive is True
        assert input_widget.filter_mode == Qt.MatchStartsWith
        assert input_widget.completion_mode == QCompleter.InlineCompletion

    def test_auto_complete_input_properties(self, qt_widget_cleanup):
        """Test des propriétés du widget."""
        input_widget = AutoCompleteInput()

        # ////// TEST SUGGESTIONS PROPERTY
        suggestions = ["item1", "item2", "item3"]
        input_widget.suggestions = suggestions
        assert input_widget.suggestions == suggestions

        # ////// TEST CASE_SENSITIVE PROPERTY
        input_widget.case_sensitive = True
        assert input_widget.case_sensitive is True

        # ////// TEST FILTER_MODE PROPERTY
        input_widget.filter_mode = Qt.MatchStartsWith
        assert input_widget.filter_mode == Qt.MatchStartsWith

        # ////// TEST COMPLETION_MODE PROPERTY
        input_widget.completion_mode = QCompleter.InlineCompletion
        assert input_widget.completion_mode == QCompleter.InlineCompletion

    def test_auto_complete_input_add_suggestion(self, qt_widget_cleanup):
        """Test de la méthode add_suggestion."""
        input_widget = AutoCompleteInput()

        # ////// ÉTAT INITIAL
        assert input_widget.suggestions == []

        # ////// AJOUTER UNE SUGGESTION
        input_widget.add_suggestion("new_item")
        assert "new_item" in input_widget.suggestions

        # ////// AJOUTER UNE AUTRE SUGGESTION
        input_widget.add_suggestion("another_item")
        assert "new_item" in input_widget.suggestions
        assert "another_item" in input_widget.suggestions
        assert len(input_widget.suggestions) == 2

    def test_auto_complete_input_remove_suggestion(self, qt_widget_cleanup):
        """Test de la méthode remove_suggestion."""
        input_widget = AutoCompleteInput(suggestions=["item1", "item2", "item3"])

        # ////// ÉTAT INITIAL
        assert len(input_widget.suggestions) == 3

        # ////// SUPPRIMER UNE SUGGESTION
        input_widget.remove_suggestion("item2")
        assert "item1" in input_widget.suggestions
        assert "item2" not in input_widget.suggestions
        assert "item3" in input_widget.suggestions
        assert len(input_widget.suggestions) == 2

    def test_auto_complete_input_clear_suggestions(self, qt_widget_cleanup):
        """Test de la méthode clear_suggestions."""
        input_widget = AutoCompleteInput(suggestions=["item1", "item2", "item3"])

        # ////// ÉTAT INITIAL
        assert len(input_widget.suggestions) == 3

        # ////// EFFACER TOUTES LES SUGGESTIONS
        input_widget.clear_suggestions()
        assert input_widget.suggestions == []

    def test_auto_complete_input_refresh_style(self, qt_widget_cleanup):
        """Test de la méthode refresh_style."""
        input_widget = AutoCompleteInput()

        # ////// LA MÉTHODE NE DOIT PAS LEVER D'EXCEPTION
        try:
            input_widget.refresh_style()
        except Exception as e:
            pytest.fail(f"refresh_style() a levé une exception: {e}")

    def test_auto_complete_input_completer_integration(self, qt_widget_cleanup):
        """Test de l'intégration avec QCompleter."""
        input_widget = AutoCompleteInput(suggestions=["test1", "test2"])

        # ////// VÉRIFIER QUE LE COMPLETER EST CONFIGURÉ
        assert input_widget.completer() is not None
        assert isinstance(input_widget.completer(), QCompleter)

        # ////// VÉRIFIER QUE LE MODÈLE CONTIENT LES SUGGESTIONS
        model = input_widget.completer().model()
        assert model is not None
        assert model.rowCount() == 2

    def test_auto_complete_input_case_sensitivity(self, qt_widget_cleanup):
        """Test de la sensibilité à la casse."""
        input_widget = AutoCompleteInput(suggestions=["Test", "test", "TEST"])

        # ////// TEST CASE INSENSITIVE (DÉFAUT)
        input_widget.case_sensitive = False
        assert input_widget.case_sensitive is False

        # ////// TEST CASE SENSITIVE
        input_widget.case_sensitive = True
        assert input_widget.case_sensitive is True

    def test_auto_complete_input_filter_modes(self, qt_widget_cleanup):
        """Test des différents modes de filtrage."""
        input_widget = AutoCompleteInput()

        # ////// TEST MATCHCONTAINS (DÉFAUT)
        input_widget.filter_mode = Qt.MatchContains
        assert input_widget.filter_mode == Qt.MatchContains

        # ////// TEST MATCHSTARTSWITH
        input_widget.filter_mode = Qt.MatchStartsWith
        assert input_widget.filter_mode == Qt.MatchStartsWith

        # ////// TEST MATCHENDSWITH
        input_widget.filter_mode = Qt.MatchEndsWith
        assert input_widget.filter_mode == Qt.MatchEndsWith

    def test_auto_complete_input_completion_modes(self, qt_widget_cleanup):
        """Test des différents modes de complétion."""
        input_widget = AutoCompleteInput()

        # ////// TEST POPUPCOMPLETION (DÉFAUT)
        input_widget.completion_mode = QCompleter.PopupCompletion
        assert input_widget.completion_mode == QCompleter.PopupCompletion

        # ////// TEST INLINECOMPLETION
        input_widget.completion_mode = QCompleter.InlineCompletion
        assert input_widget.completion_mode == QCompleter.InlineCompletion

        # ////// TEST UNFILTEREDPOPUPCOMPLETION
        input_widget.completion_mode = QCompleter.UnfilteredPopupCompletion
        assert input_widget.completion_mode == QCompleter.UnfilteredPopupCompletion

    def test_auto_complete_input_text_handling(self, qt_widget_cleanup):
        """Test de la gestion du texte."""
        input_widget = AutoCompleteInput()

        # ////// TEST SETTEXT
        input_widget.setText("test text")
        assert input_widget.text() == "test text"

        # ////// TEST CLEAR
        input_widget.clear()
        assert input_widget.text() == ""

        # ////// TEST PLACEHOLDER
        input_widget.setPlaceholderText("Enter text...")
        assert input_widget.placeholderText() == "Enter text..."

    def test_auto_complete_input_multiple_suggestions(self, qt_widget_cleanup):
        """Test avec de nombreuses suggestions."""
        suggestions = [f"item_{i}" for i in range(100)]
        input_widget = AutoCompleteInput(suggestions=suggestions)

        # ////// VÉRIFIER QUE TOUTES LES SUGGESTIONS SONT PRÉSENTES
        assert len(input_widget.suggestions) == 100
        assert "item_0" in input_widget.suggestions
        assert "item_99" in input_widget.suggestions

        # ////// VÉRIFIER QUE LE COMPLETER PEUT GÉRER BEAUCOUP D'ÉLÉMENTS
        assert input_widget.completer().model().rowCount() == 100

    def test_auto_complete_input_empty_suggestions(self, qt_widget_cleanup):
        """Test avec des suggestions vides."""
        input_widget = AutoCompleteInput(suggestions=[])

        # ////// VÉRIFIER L'ÉTAT INITIAL
        assert input_widget.suggestions == []

        # ////// AJOUTER DES SUGGESTIONS
        input_widget.add_suggestion("new_item")
        assert input_widget.suggestions == ["new_item"]

        # ////// EFFACER ET VÉRIFIER
        input_widget.clear_suggestions()
        assert input_widget.suggestions == []

    def test_auto_complete_input_duplicate_suggestions(self, qt_widget_cleanup):
        """Test avec des suggestions en double."""
        input_widget = AutoCompleteInput()

        # ////// AJOUTER DES SUGGESTIONS EN DOUBLE
        input_widget.add_suggestion("item")
        input_widget.add_suggestion("item")
        input_widget.add_suggestion("item")

        # ////// VÉRIFIER QUE LES DOUBLONS SONT IGNORÉS (comportement du widget)
        assert input_widget.suggestions.count("item") == 1

        # ////// SUPPRIMER L'OCCURRENCE
        input_widget.remove_suggestion("item")
        assert input_widget.suggestions.count("item") == 0

    def test_auto_complete_input_special_characters(self, qt_widget_cleanup):
        """Test avec des caractères spéciaux."""
        special_suggestions = [
            "test@example.com",
            "user-name_123",
            "file/path/to/item",
            "item with spaces",
            "item\nwith\nnewlines",
            "item\twith\ttabs",
        ]

        input_widget = AutoCompleteInput(suggestions=special_suggestions)

        # ////// VÉRIFIER QUE TOUTES LES SUGGESTIONS SONT PRÉSERVÉES
        assert len(input_widget.suggestions) == 6
        for suggestion in special_suggestions:
            assert suggestion in input_widget.suggestions

    def test_auto_complete_input_property_type(self, qt_widget_cleanup):
        """Test de la propriété type pour QSS."""
        input_widget = AutoCompleteInput()

        # ////// VÉRIFIER QUE LA PROPRIÉTÉ TYPE EST DÉFINIE
        # Note: AutoCompleteInput hérite de QLineEdit, donc pas de propriété type personnalisée
        # Mais on peut vérifier que le widget fonctionne correctement
        assert input_widget is not None
        assert isinstance(input_widget, AutoCompleteInput)



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# FRAMED LABEL
from .framed_label import *

# INDICATOR LABEL
from .indicator_label import *

# HOVER LABEL
from .hover_label import *

# CLICKABLE TAG LABEL
from .clickable_tag_label import *


# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
import re

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Signal,
    QTimer,
    Qt,
    QSize,
)
from PySide6.QtGui import (
    QPainter,
    QPen,
    QColor,
    QMouseEvent,
    QPaintEvent,
)
from PySide6.QtWidgets import (
    QWidget,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////

# ////// TYPE HINTS IMPROVEMENTS FOR PYSIDE6 6.9.1
from typing import Optional, Union, Literal

# UTILITY FUNCTIONS
# ///////////////////////////////////////////////////////////////


def parse_css_color(color_str: Union[QColor, str]) -> QColor:
    """Parse CSS color strings (rgb, rgba, hex, named colors) to QColor."""
    if isinstance(color_str, QColor):
        return color_str

    color_str = str(color_str).strip()

    # Parse rgb(r, g, b)
    rgb_match = re.match(r"rgb\((\d+),\s*(\d+),\s*(\d+)\)", color_str)
    if rgb_match:
        r, g, b = map(int, rgb_match.groups())
        return QColor(r, g, b)

    # Parse rgba(r, g, b, a)
    rgba_match = re.match(r"rgba\((\d+),\s*(\d+),\s*(\d+),\s*([\d.]+)\)", color_str)
    if rgba_match:
        r, g, b, a = rgba_match.groups()
        r, g, b = map(int, [r, g, b])
        a = float(a) * 255  # Convert 0-1 to 0-255
        return QColor(r, g, b, int(a))

    # Fallback to QColor constructor (hex, named colors, etc.)
    return QColor(color_str)


# CLASS
# ///////////////////////////////////////////////////////////////


class CircularTimer(QWidget):
    """
    CircularTimer est un timer circulaire animé pour indiquer une progression ou un temps écoulé.

    Parameters
    ----------
    parent : QWidget, optional
        Parent Qt (default: None).
    duration : int, optional
        Durée totale de l'animation en millisecondes (par défaut 5000).
    ring_color : QColor | str, optional
        Couleur de l'arc de progression (par défaut #0078d4).
        Supporte: hex (#ff0000), rgb(255,0,0), rgba(255,0,0,0.5), noms (red).
    node_color : QColor | str, optional
        Couleur du centre (par défaut #2d2d2d).
        Supporte: hex (#ffffff), rgb(255,255,255), rgba(255,255,255,0.8), noms (white).
    ring_width_mode : str, optional
        "small", "medium" (défaut), ou "large". Contrôle dynamiquement l'épaisseur de l'arc.
    pen_width : int | float, optional
        Épaisseur de l'arc (prioritaire sur ring_width_mode si défini).
    loop : bool, optional
        Si True, le timer boucle automatiquement à chaque cycle (par défaut False).
    *args, **kwargs :
        Additional arguments passed to QWidget.

    Properties
    ----------
    duration : int
        Durée totale de l'animation.
    elapsed : int
        Temps écoulé depuis le début de l'animation.
    running : bool
        Indique si le timer est en cours d'animation.
    ring_color : QColor
        Couleur de l'arc de progression.
    node_color : QColor
        Couleur du centre.
    ring_width_mode : str
        "small", "medium", "large". Contrôle dynamiquement l'épaisseur de l'arc.
    pen_width : float
        Épaisseur de l'arc (prioritaire sur ring_width_mode).
    loop : bool
        Si True, le timer boucle automatiquement à chaque cycle.

    Signals
    -------
    timerReset()
        Émis lorsque le timer est réinitialisé.
    clicked()
        Émis lors d'un clic sur le widget.
    cycleCompleted()
        Émis à chaque fin de cycle (même si loop=False).
    """

    timerReset = Signal()
    clicked = Signal()
    cycleCompleted = Signal()

    # INITIALIZATION
    # ///////////////////////////////////////////////////////////////

    def __init__(
        self,
        parent=None,
        duration: int = 5000,
        ring_color: Union[QColor, str] = "#0078d4",
        node_color: Union[QColor, str] = "#2d2d2d",
        ring_width_mode: Literal["small", "medium", "large"] = "medium",
        pen_width: Optional[Union[int, float]] = None,
        loop: bool = False,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(parent, *args, **kwargs)
        self.setProperty("type", "CircularTimer")

        # ////// INITIALIZE PROPERTIES
        self._duration: int = duration
        self._elapsed: int = 0
        self._running: bool = False
        self._ring_color: QColor = parse_css_color(ring_color)
        self._node_color: QColor = parse_css_color(node_color)
        self._ring_width_mode: str = ring_width_mode
        self._pen_width: Optional[float] = pen_width
        self._loop: bool = bool(loop)
        self._last_update = None
        self._interval: int = 16  # ~60 FPS

        # ////// SETUP TIMER
        self.timer = QTimer(self)
        self.timer.timeout.connect(self._on_timer)

    # PROPERTIES
    # ///////////////////////////////////////////////////////////////

    @property
    def duration(self) -> int:
        """Get the total duration."""
        return self._duration

    @duration.setter
    def duration(self, value: int) -> None:
        """Set the total duration."""
        self._duration = int(value)
        self.update()

    @property
    def elapsed(self) -> int:
        """Get the elapsed time."""
        return self._elapsed

    @elapsed.setter
    def elapsed(self, value: int) -> None:
        """Set the elapsed time."""
        self._elapsed = int(value)
        self.update()

    @property
    def running(self) -> bool:
        """Get whether the timer is running."""
        return self._running

    @property
    def ring_color(self) -> QColor:
        """Get the ring color."""
        return self._ring_color

    @ring_color.setter
    def ring_color(self, value: Union[QColor, str]) -> None:
        """Set the ring color."""
        self._ring_color = parse_css_color(value)
        self.update()

    @property
    def node_color(self) -> QColor:
        """Get the node color."""
        return self._node_color

    @node_color.setter
    def node_color(self, value: Union[QColor, str]) -> None:
        """Set the node color."""
        self._node_color = parse_css_color(value)
        self.update()

    @property
    def ring_width_mode(self) -> str:
        """Get the ring width mode."""
        return self._ring_width_mode

    @ring_width_mode.setter
    def ring_width_mode(self, value: str) -> None:
        """Set the ring width mode."""
        if value not in ("small", "medium", "large"):
            value = "medium"
        self._ring_width_mode = value
        self.update()

    @property
    def pen_width(self) -> Optional[float]:
        """Get the pen width."""
        return self._pen_width

    @pen_width.setter
    def pen_width(self, value: Optional[Union[int, float]]) -> None:
        """Set the pen width."""
        self._pen_width = float(value) if value is not None else None
        self.update()

    @property
    def loop(self) -> bool:
        """Get whether the timer loops."""
        return self._loop

    @loop.setter
    def loop(self, value: bool) -> None:
        """Set whether the timer loops."""
        self._loop = bool(value)

    # EVENT FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def mousePressEvent(self, event: QMouseEvent) -> None:
        """Handle mouse press events."""
        # ////// HANDLE CLICK
        self.clicked.emit()
        # //////

    # UTILITY FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def startTimer(self) -> None:
        """Démarre le timer circulaire."""
        # ////// START TIMER
        self.stopTimer()  # Toujours arrêter avant de démarrer
        self._running = True
        self._last_update = None
        self.timer.start(self._interval)
        # //////

    def stopTimer(self) -> None:
        """Arrête le timer circulaire."""
        # ////// STOP TIMER
        self.resetTimer()  # Toujours repartir de zéro
        self._running = False
        self.timer.stop()
        # //////

    def resetTimer(self) -> None:
        """Réinitialise le timer circulaire."""
        # ////// RESET TIMER
        self._elapsed = 0
        self._last_update = None
        self.timerReset.emit()
        self.update()
        # //////

    def _on_timer(self) -> None:
        """Callback interne pour l'animation fluide."""
        # ////// TIMER CALLBACK
        import time

        now = time.monotonic() * 1000  # ms
        if self._last_update is None:
            self._last_update = now
            return
        delta = now - self._last_update
        self._last_update = now
        self._elapsed += delta
        if self._elapsed > self._duration:
            self.cycleCompleted.emit()
            if self._loop:
                self.resetTimer()
                self._running = True
                self._last_update = now
                # Le timer continue (pas d'arrêt)
            else:
                self.resetTimer()
                self.stopTimer()
        self.update()
        # //////

    # OVERRIDE FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def minimumSizeHint(self) -> QSize:
        """Taille minimale recommandée pour le widget."""
        return QSize(24, 24)

    def paintEvent(self, event: QPaintEvent) -> None:
        """Dessine le timer circulaire animé."""
        # ////// PAINT EVENT
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        size = min(self.width(), self.height())

        # ////// PEN WIDTH (mode dynamique ou valeur fixe)
        if self._pen_width is not None:
            penWidth = self._pen_width
        else:
            if self._ring_width_mode == "small":
                penWidth = max(size * 0.12, 3)
            elif self._ring_width_mode == "large":
                penWidth = max(size * 0.28, 3)
            else:  # medium
                penWidth = max(size * 0.18, 3)

        # ////// NODE CIRCLE (centrage précis)
        center = size / 2
        node_radius = (size - 2 * penWidth) / 2 - penWidth / 2
        if node_radius > 0:
            painter.setPen(Qt.NoPen)
            painter.setBrush(self._node_color)
            painter.drawEllipse(
                center - node_radius,
                center - node_radius,
                2 * node_radius,
                2 * node_radius,
            )

        # ////// RING ARC (sens horaire, départ 12h)
        painter.setPen(QPen(self._ring_color, penWidth, Qt.SolidLine, Qt.RoundCap))
        angle = int((self._elapsed / self._duration) * 360 * 16)
        painter.drawArc(
            penWidth,
            penWidth,
            size - 2 * penWidth,
            size - 2 * penWidth,
            90 * 16,
            -angle,  # sens horaire
        )
        # //////

    # STYLE FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def refresh_style(self) -> None:
        """Refresh the widget's style (useful after dynamic stylesheet changes)."""
        self.style().unpolish(self)
        self.style().polish(self)
        self.update()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
Exemple d'utilisation des widgets misc EzQt_Widgets.

Ce script démontre l'utilisation de tous les types de widgets misc disponibles :
- OptionSelector
- CircularTimer
- ToggleIcon
- ToggleSwitch
- DraggableList
"""

import sys
import os
import re
import yaml
from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QFont, QPixmap, QIcon
from PySide6.QtWidgets import (
    QApplication,
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QLabel,
    QPushButton,
    QGroupBox,
    QMessageBox,
    QScrollArea,
)

# Import des widgets EzQt
from ezqt_widgets.misc import (
    OptionSelector,
    CircularTimer,
    ToggleIcon,
    ToggleSwitch,
    DraggableList,
)


def load_and_apply_qss(app, qss_path, yaml_path, theme_name="dark"):
    """Fonction utilitaire pour charger et appliquer le QSS avec variables."""
    try:
        # Charger les variables du thème depuis le YAML
        with open(yaml_path, encoding="utf-8") as f:
            config = yaml.safe_load(f)
        theme_vars = config["theme_palette"][theme_name]

        # Charger le QSS
        with open(qss_path, encoding="utf-8") as f:
            qss = f.read()

        # Remplacer les variables $ par leur valeur
        def repl(match):
            var = match.group(0)
            return theme_vars.get(var, var)

        qss = re.sub(r"\$_[a-zA-Z0-9_]+", repl, qss)
        app.setStyleSheet(qss)
        print(f"Thème '{theme_name}' chargé avec succès")

    except Exception as e:
        print(f"Erreur lors du chargement du thème: {e}")
        # Appliquer un style par défaut en cas d'erreur
        app.setStyleSheet("")


class MiscExampleWidget(QWidget):
    """Widget de démonstration pour tous les widgets misc."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Exemples Misc - EzQt_Widgets")
        self.setMinimumSize(900, 800)
        self.setup_ui()

    def setup_ui(self):
        """Configure l'interface utilisateur."""
        # Layout principal
        main_layout = QVBoxLayout(self)
        main_layout.setSpacing(0)
        main_layout.setContentsMargins(0, 0, 0, 0)

        # Création du ScrollArea
        scroll_area = QScrollArea()
        scroll_area.setWidgetResizable(True)
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        scroll_area.setFrameShape(QScrollArea.NoFrame)

        # Widget conteneur pour le contenu
        content_widget = QWidget()
        content_layout = QVBoxLayout(content_widget)
        content_layout.setSpacing(20)
        content_layout.setContentsMargins(20, 20, 20, 20)

        # Titre
        title = QLabel("Exemples Misc - EzQt_Widgets")
        title.setFont(QFont("Segoe UI", 16, QFont.Bold))
        title.setAlignment(Qt.AlignCenter)
        title.setStyleSheet("color: #2c3e50; margin-bottom: 20px;")
        content_layout.addWidget(title)

        # Groupe OptionSelector
        option_group = QGroupBox("OptionSelector")
        option_layout = QVBoxLayout(option_group)

        option_label = QLabel("Sélecteur d'options avec différentes configurations:")
        option_layout.addWidget(option_label)

        # Création d'un sélecteur d'options
        options = ["Option 1", "Option 2", "Option 3", "Option 4", "Option 5"]
        self.option_selector = OptionSelector(options, default_id=0)
        self.option_selector.valueChanged.connect(self.on_option_changed)
        option_layout.addWidget(self.option_selector)

        self.option_output = QLabel("Option sélectionnée: Option 1")
        self.option_output.setStyleSheet("font-weight: bold; color: #4CAF50;")
        option_layout.addWidget(self.option_output)

        content_layout.addWidget(option_group)

        # Groupe CircularTimer
        timer_group = QGroupBox("CircularTimer")
        timer_layout = QVBoxLayout(timer_group)

        timer_label = QLabel("Timer circulaire avec contrôle:")
        timer_layout.addWidget(timer_label)

        # Layout horizontal pour le timer et les contrôles
        timer_control_layout = QHBoxLayout()

        self.circular_timer = CircularTimer(
            duration=10000
        )  # 10 secondes (en millisecondes)
        timer_control_layout.addWidget(self.circular_timer)

        # Contrôles pour le timer
        timer_buttons_layout = QVBoxLayout()

        start_button = QPushButton("Démarrer")
        start_button.clicked.connect(self.start_timer)
        timer_buttons_layout.addWidget(start_button)

        stop_button = QPushButton("Arrêter")
        stop_button.clicked.connect(self.stop_timer)
        timer_buttons_layout.addWidget(stop_button)

        reset_button = QPushButton("Réinitialiser")
        reset_button.clicked.connect(self.reset_timer)
        timer_buttons_layout.addWidget(reset_button)

        timer_control_layout.addLayout(timer_buttons_layout)
        timer_layout.addLayout(timer_control_layout)

        self.timer_status = QLabel("État: Arrêté")
        timer_layout.addWidget(self.timer_status)

        # Connexion des signaux du timer
        self.circular_timer.cycleCompleted.connect(self.on_timer_timeout)
        # Mise à jour manuelle du statut car CircularTimer n'a pas de signaux started/stopped

        content_layout.addWidget(timer_group)

        # Groupe ToggleIcon
        toggle_icon_group = QGroupBox("ToggleIcon")
        toggle_icon_layout = QVBoxLayout(toggle_icon_group)

        toggle_icon_label = QLabel("Icône basculante:")
        toggle_icon_layout.addWidget(toggle_icon_label)

        # Création d'icônes simples
        pixmap1 = QPixmap(32, 32)
        pixmap1.fill(Qt.green)
        icon1 = QIcon(pixmap1)

        pixmap2 = QPixmap(32, 32)
        pixmap2.fill(Qt.red)
        icon2 = QIcon(pixmap2)

        self.toggle_icon = ToggleIcon(opened_icon=icon1, closed_icon=icon2)
        self.toggle_icon.setToolTip("Cliquez pour basculer")
        self.toggle_icon.clicked.connect(self.on_toggle_icon_clicked)
        toggle_icon_layout.addWidget(self.toggle_icon)

        self.toggle_icon_status = QLabel("État: Icône 1")
        toggle_icon_layout.addWidget(self.toggle_icon_status)

        content_layout.addWidget(toggle_icon_group)

        # Groupe ToggleSwitch
        toggle_switch_group = QGroupBox("ToggleSwitch")
        toggle_switch_layout = QVBoxLayout(toggle_switch_group)

        toggle_switch_label = QLabel("Interrupteur basculant:")
        toggle_switch_layout.addWidget(toggle_switch_label)

        self.toggle_switch = ToggleSwitch(checked=False)
        self.toggle_switch.toggled.connect(self.on_toggle_switch_changed)
        toggle_switch_layout.addWidget(self.toggle_switch)

        self.toggle_switch_status = QLabel("État: Désactivé")
        toggle_switch_layout.addWidget(self.toggle_switch_status)

        content_layout.addWidget(toggle_switch_group)

        # Groupe DraggableList
        item_list_group = QGroupBox("DraggableList")
        item_list_layout = QVBoxLayout(item_list_group)

        item_list_label = QLabel("Liste d'éléments réorganisables avec drag & drop:")
        item_list_layout.addWidget(item_list_label)

        # Création de la liste d'éléments
        initial_items = [
            "Premier élément",
            "Deuxième élément", 
            "Troisième élément",
            "Quatrième élément",
        ]

        # Liste en mode normal
        self.item_list = DraggableList(
            items=initial_items,
            icon_color="#FF4444",
            max_height=200,
            min_width=120
        )
        self.item_list.itemRemoved.connect(self.on_item_removed)
        self.item_list.itemMoved.connect(self.on_item_moved)
        self.item_list.orderChanged.connect(self.on_order_changed)
        item_list_layout.addWidget(self.item_list)

        # Liste en mode compact
        compact_label = QLabel("Liste en mode compact:")
        item_list_layout.addWidget(compact_label)

        self.compact_list = DraggableList(
            items=["Option A", "Option B", "Option C"],
            compact=True,
            icon_color="grey",
            max_height=150
        )
        self.compact_list.itemRemoved.connect(self.on_item_removed)
        self.compact_list.itemMoved.connect(self.on_item_moved)
        item_list_layout.addWidget(self.compact_list)

        # Contrôles pour la liste
        item_controls_layout = QHBoxLayout()

        add_button = QPushButton("Ajouter")
        add_button.clicked.connect(self.add_item)
        item_controls_layout.addWidget(add_button)

        clear_button = QPushButton("Vider")
        clear_button.clicked.connect(self.item_list.clear_items)
        item_controls_layout.addWidget(clear_button)

        compact_toggle = QPushButton("Mode Compact")
        compact_toggle.setCheckable(True)
        compact_toggle.clicked.connect(self.toggle_compact_mode)
        item_controls_layout.addWidget(compact_toggle)

        item_list_layout.addLayout(item_controls_layout)

        self.item_list_status = QLabel("Éléments: 4 | Mode: Normal")
        item_list_layout.addWidget(self.item_list_status)

        content_layout.addWidget(item_list_group)

        # Boutons de test
        test_group = QGroupBox("Tests Interactifs")
        test_layout = QHBoxLayout(test_group)

        test_option_btn = QPushButton("Test Option")
        test_option_btn.clicked.connect(self.test_option_selector)
        test_layout.addWidget(test_option_btn)

        test_timer_btn = QPushButton("Test Timer")
        test_timer_btn.clicked.connect(self.test_circular_timer)
        test_layout.addWidget(test_timer_btn)

        test_toggle_btn = QPushButton("Test Toggle")
        test_toggle_btn.clicked.connect(self.test_toggle_widgets)
        test_layout.addWidget(test_toggle_btn)

        test_item_btn = QPushButton("Test Items")
        test_item_btn.clicked.connect(self.test_item_list)
        test_layout.addWidget(test_item_btn)

        reset_btn = QPushButton("Reset")
        reset_btn.clicked.connect(self.reset_all)
        test_layout.addWidget(reset_btn)

        content_layout.addWidget(test_group)

        # Espacement en bas pour éviter que le contenu soit coupé
        content_layout.addStretch()

        # Configuration du ScrollArea
        scroll_area.setWidget(content_widget)
        main_layout.addWidget(scroll_area)

        # Compteur pour les éléments
        self.item_counter = 5

    def on_option_changed(self, option):
        """Callback appelé quand une option est sélectionnée."""
        self.option_output.setText(f"Option sélectionnée: {option}")
        print(f"Option sélectionnée: {option}")

    def start_timer(self):
        """Démarre le timer."""
        self.circular_timer.startTimer()
        self.timer_status.setText("État: En cours")
        print("Timer démarré!")

    def stop_timer(self):
        """Arrête le timer."""
        self.circular_timer.stopTimer()
        self.timer_status.setText("État: Arrêté")
        print("Timer arrêté!")

    def reset_timer(self):
        """Réinitialise le timer."""
        self.circular_timer.resetTimer()
        self.timer_status.setText("État: Arrêté")
        print("Timer réinitialisé!")

    def on_timer_timeout(self):
        """Callback appelé quand le timer expire."""
        self.timer_status.setText("État: Terminé")
        print("Timer terminé!")

    def on_toggle_icon_clicked(self):
        """Callback appelé quand l'icône basculante est cliquée."""
        current_state = "Icône 2" if self.toggle_icon.state == "opened" else "Icône 1"
        self.toggle_icon_status.setText(f"État: {current_state}")
        print(f"Icône basculée vers: {current_state}")

    def on_toggle_switch_changed(self, checked):
        """Callback appelé quand l'interrupteur bascule."""
        state = "Activé" if checked else "Désactivé"
        self.toggle_switch_status.setText(f"État: {state}")
        print(f"Interrupteur: {state}")

    def on_item_removed(self, item_id, position):
        """Callback appelé quand un élément est supprimé."""
        print(f"Élément supprimé: {item_id} à la position {position}")
        self.update_item_count()

    def on_item_moved(self, item_id, old_pos, new_pos):
        """Callback appelé quand un élément est déplacé."""
        print(f"Élément déplacé: {item_id} de {old_pos} à {new_pos}")

    def on_order_changed(self, new_order):
        """Callback appelé quand l'ordre des éléments change."""
        print(f"Nouvel ordre: {new_order}")

    def add_item(self):
        """Ajoute un nouvel élément à la liste."""
        new_item = f"Élément {self.item_counter}"
        self.item_list.add_item(new_item, new_item)
        self.item_counter += 1
        self.update_item_count()
        print(f"Élément ajouté: {new_item}")

    def toggle_compact_mode(self):
        """Bascule entre le mode normal et compact."""
        is_compact = self.item_list.compact
        self.item_list.compact = not is_compact
        self.compact_list.compact = not is_compact
        
        mode = "Compact" if not is_compact else "Normal"
        self.update_item_count()
        print(f"Mode changé vers: {mode}")

    def update_item_count(self):
        """Met à jour le compteur d'éléments."""
        count = len(self.item_list.items)
        mode = "Compact" if self.item_list.compact else "Normal"
        self.item_list_status.setText(f"Éléments: {count} | Mode: {mode}")

    def test_option_selector(self):
        """Teste l'OptionSelector."""
        # Change vers l'option suivante
        current_id = self.option_selector.value_id
        next_id = (current_id + 1) % len(self.option_selector.options)
        self.option_selector.value_id = next_id
        print(f"Test: Option changée vers l'index {next_id}")

    def test_circular_timer(self):
        """Teste le CircularTimer."""
        if not self.circular_timer.running:
            self.start_timer()
        else:
            self.stop_timer()
        print("Test: Timer basculé")

    def test_toggle_widgets(self):
        """Teste les widgets de basculement."""
        # Bascule l'icône
        self.toggle_icon.click()
        # Bascule le switch
        self.toggle_switch.checked = not self.toggle_switch.checked
        print("Test: Widgets de basculement testés")

    def test_item_list(self):
        """Teste la DraggableList."""
        # Ajoute un élément de test
        self.add_item()
        # Bascule le mode compact
        self.toggle_compact_mode()
        print("Test: DraggableList testée (ajout + mode compact)")

    def reset_all(self):
        """Remet tout à zéro."""
        # Reset OptionSelector
        self.option_selector.value_id = 0
        self.option_output.setText("Option sélectionnée: Option 1")

        # Reset CircularTimer
        self.stop_timer()
        self.reset_timer()

        # Reset ToggleIcon
        self.toggle_icon_status.setText("État: Icône 1")

        # Reset ToggleSwitch
        self.toggle_switch.checked = False
        self.toggle_switch_status.setText("État: Désactivé")

        # Reset DraggableList
        initial_items = [
            "Premier élément",
            "Deuxième élément", 
            "Troisième élément",
            "Quatrième élément",
        ]
        self.item_list.items = initial_items
        self.item_list.compact = False
        self.compact_list.items = ["Option A", "Option B", "Option C"]
        self.compact_list.compact = True
        self.item_counter = 5
        self.update_item_count()

        print("Reset: Tous les widgets remis à zéro")


def main():
    """Fonction principale pour l'exécution individuelle."""
    app = QApplication(sys.argv)

    # Charger le thème depuis les fichiers
    bin_dir = os.path.join(os.path.dirname(__file__), "bin")
    qss_path = os.path.join(bin_dir, "main_theme.qss")
    yaml_path = os.path.join(bin_dir, "app.yaml")

    if os.path.exists(qss_path) and os.path.exists(yaml_path):
        load_and_apply_qss(app, qss_path, yaml_path, theme_name="dark")
    else:
        # Style par défaut si les fichiers de thème ne sont pas trouvés
        app.setStyleSheet(
            """
            QGroupBox {
                font-weight: bold;
                border: 2px solid #cccccc;
                border-radius: 5px;
                margin-top: 1ex;
                padding-top: 10px;
            }
            QPushButton {
                background-color: #4CAF50;
                border: none;
                color: white;
                padding: 8px 16px;
                text-align: center;
                font-size: 14px;
                border-radius: 4px;
            }
            QPushButton:hover {
                background-color: #45a049;
            }
            QLabel {
                color: #2c3e50;
            }
            QScrollArea {
                border: none;
                background-color: transparent;
            }
        """
        )

    window = MiscExampleWidget()
    window.show()

    sys.exit(app.exec())


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
Tests unitaires pour le widget LoaderButton.
"""

import pytest
from unittest.mock import patch, MagicMock
from PySide6.QtCore import QSize, Qt, QTimer
from PySide6.QtGui import QIcon, QPixmap, QMouseEvent
from PySide6.QtWidgets import QApplication

from ezqt_widgets.button.loader_button import (
    LoaderButton,
    create_spinner_pixmap,
    create_loading_icon,
    create_success_icon,
    create_error_icon,
)


pytestmark = pytest.mark.unit


class TestUtilityFunctions:
    """Tests pour les fonctions utilitaires."""

    def test_create_spinner_pixmap(self, qt_widget_cleanup):
        """Test de create_spinner_pixmap."""
        pixmap = create_spinner_pixmap(16, "#0078d4")

        assert pixmap is not None
        assert isinstance(pixmap, QPixmap)
        assert pixmap.size() == QSize(16, 16)
        assert not pixmap.isNull()

    def test_create_spinner_pixmap_custom_size(self, qt_widget_cleanup):
        """Test de create_spinner_pixmap avec taille personnalisée."""
        pixmap = create_spinner_pixmap(32, "#FF0000")

        assert pixmap.size() == QSize(32, 32)

    def test_create_loading_icon(self, qt_widget_cleanup):
        """Test de create_loading_icon."""
        icon = create_loading_icon(16, "#0078d4")

        assert icon is not None
        assert isinstance(icon, QIcon)
        assert not icon.isNull()

    def test_create_success_icon(self, qt_widget_cleanup):
        """Test de create_success_icon."""
        icon = create_success_icon(16, "#28a745")

        assert icon is not None
        assert isinstance(icon, QIcon)
        assert not icon.isNull()

    def test_create_error_icon(self, qt_widget_cleanup):
        """Test de create_error_icon."""
        icon = create_error_icon(16, "#dc3545")

        assert icon is not None
        assert isinstance(icon, QIcon)
        assert not icon.isNull()


class TestLoaderButton:
    """Tests pour la classe LoaderButton."""

    def test_loader_button_creation_default(self, qt_widget_cleanup):
        """Test de création avec paramètres par défaut."""
        button = LoaderButton()

        assert button is not None
        assert isinstance(button, LoaderButton)
        assert button.text == ""
        assert button.loading_text == "Chargement..."
        assert button.animation_speed == 100
        assert button.auto_reset is True
        assert button.success_display_time == 1000
        assert button.error_display_time == 2000
        assert not button.is_loading

    def test_loader_button_creation_with_parameters(self, qt_widget_cleanup):
        """Test de création avec paramètres personnalisés."""
        button = LoaderButton(
            text="Test Button",
            loading_text="Loading...",
            animation_speed=200,
            auto_reset=False,
            success_display_time=2000,
            error_display_time=3000,
        )

        assert button.text == "Test Button"
        assert button.loading_text == "Loading..."
        assert button.animation_speed == 200
        assert button.auto_reset is False
        assert button.success_display_time == 2000
        assert button.error_display_time == 3000

    def test_loader_button_properties(self, qt_widget_cleanup):
        """Test des propriétés du bouton."""
        button = LoaderButton()

        # ////// TEST TEXT PROPERTY
        button.text = "New Text"
        assert button.text == "New Text"

        # ////// TEST ICON PROPERTY
        pixmap = QPixmap(16, 16)
        pixmap.fill(Qt.red)
        icon = QIcon(pixmap)
        button.icon = icon
        assert button.icon is not None

        # ////// TEST LOADING_TEXT PROPERTY
        button.loading_text = "Custom Loading"
        assert button.loading_text == "Custom Loading"

        # ////// TEST LOADING_ICON PROPERTY
        button.loading_icon = icon
        assert button.loading_icon is not None

        # ////// TEST SUCCESS_ICON PROPERTY
        button.success_icon = icon
        assert button.success_icon is not None

        # ////// TEST ERROR_ICON PROPERTY
        button.error_icon = icon
        assert button.error_icon is not None

        # ////// TEST ANIMATION_SPEED PROPERTY
        button.animation_speed = 150
        assert button.animation_speed == 150

        # ////// TEST AUTO_RESET PROPERTY
        button.auto_reset = False
        assert button.auto_reset is False

        # ////// TEST SUCCESS_DISPLAY_TIME PROPERTY
        button.success_display_time = 1500
        assert button.success_display_time == 1500

        # ////// TEST ERROR_DISPLAY_TIME PROPERTY
        button.error_display_time = 2500
        assert button.error_display_time == 2500

    def test_loader_button_signals(self, qt_widget_cleanup):
        """Test des signaux du bouton."""
        button = LoaderButton()

        # ////// TEST LOADINGSTARTED SIGNAL
        signal_started = False

        def on_loading_started():
            nonlocal signal_started
            signal_started = True

        button.loadingStarted.connect(on_loading_started)
        button.start_loading()

        # ////// VÉRIFIER QUE LE SIGNAL A ÉTÉ ÉMIS
        # Note: Dans un contexte de test, les signaux peuvent ne pas être émis immédiatement
        # Vérifions plutôt que start_loading() fonctionne
        assert button.is_loading

        # ////// TEST LOADINGFINISHED SIGNAL
        signal_finished = False

        def on_loading_finished():
            nonlocal signal_finished
            signal_finished = True

        button.loadingFinished.connect(on_loading_finished)
        button.stop_loading(success=True)

        # ////// VÉRIFIER QUE LE SIGNAL A ÉTÉ ÉMIS
        # Vérifions plutôt que stop_loading() fonctionne
        assert not button.is_loading

        # ////// TEST LOADINGFAILED SIGNAL
        signal_failed = False
        error_message = ""

        def on_loading_failed(message):
            nonlocal signal_failed, error_message
            signal_failed = True
            error_message = message

        button.loadingFailed.connect(on_loading_failed)
        button.stop_loading(success=False, error_message="Test error")

        # ////// VÉRIFIER QUE LE SIGNAL A ÉTÉ ÉMIS
        # Vérifions plutôt que stop_loading() avec erreur fonctionne
        assert not button.is_loading

    def test_loader_button_start_loading(self, qt_widget_cleanup):
        """Test de la méthode start_loading."""
        button = LoaderButton()

        # ////// VÉRIFIER L'ÉTAT INITIAL
        assert not button.is_loading

        # ////// DÉMARRER LE CHARGEMENT
        button.start_loading()

        # ////// VÉRIFIER L'ÉTAT DE CHARGEMENT
        assert button.is_loading
        assert not button.isEnabled()  # Bouton désactivé pendant le chargement

    def test_loader_button_stop_loading_success(self, qt_widget_cleanup):
        """Test de stop_loading avec succès."""
        button = LoaderButton()

        # ////// DÉMARRER PUIS ARRÊTER LE CHARGEMENT
        button.start_loading()
        button.stop_loading(success=True)

        # ////// VÉRIFIER L'ÉTAT FINAL
        assert not button.is_loading
        assert button.isEnabled()  # Bouton réactivé

    def test_loader_button_stop_loading_error(self, qt_widget_cleanup):
        """Test de stop_loading avec erreur."""
        button = LoaderButton()

        # ////// DÉMARRER PUIS ARRÊTER LE CHARGEMENT AVEC ERREUR
        button.start_loading()
        button.stop_loading(success=False, error_message="Test error")

        # ////// VÉRIFIER L'ÉTAT FINAL
        assert not button.is_loading
        assert button.isEnabled()  # Bouton réactivé

    def test_loader_button_auto_reset_disabled(self, qt_widget_cleanup):
        """Test avec auto_reset désactivé."""
        button = LoaderButton(auto_reset=False)

        # ////// DÉMARRER ET ARRÊTER LE CHARGEMENT
        button.start_loading()
        button.stop_loading(success=True)

        # ////// VÉRIFIER QUE L'ÉTAT DE SUCCÈS PERSISTE
        assert not button.is_loading
        # Note: L'état de succès persiste car auto_reset=False

    def test_loader_button_size_hints(self, qt_widget_cleanup):
        """Test des méthodes de taille."""
        button = LoaderButton(text="Test Button")

        # ////// TEST SIZEHINT
        size_hint = button.sizeHint()
        assert size_hint is not None
        assert isinstance(size_hint, QSize)
        assert size_hint.width() > 0
        assert size_hint.height() > 0

        # ////// TEST MINIMUMSIZEHINT
        min_size_hint = button.minimumSizeHint()
        assert min_size_hint is not None
        assert isinstance(min_size_hint, QSize)
        assert min_size_hint.width() > 0
        assert min_size_hint.height() > 0

    def test_loader_button_refresh_style(self, qt_widget_cleanup):
        """Test de la méthode refresh_style."""
        button = LoaderButton()

        # ////// LA MÉTHODE NE DOIT PAS LEVER D'EXCEPTION
        try:
            button.refresh_style()
        except Exception as e:
            pytest.fail(f"refresh_style() a levé une exception: {e}")

    def test_loader_button_minimum_dimensions(self, qt_widget_cleanup):
        """Test des dimensions minimales."""
        button = LoaderButton(min_width=150, min_height=50)

        assert button.min_width == 150
        assert button.min_height == 50

        # ////// MODIFIER LES DIMENSIONS
        button.min_width = 200
        button.min_height = 75

        assert button.min_width == 200
        assert button.min_height == 75

        # ////// TESTER AVEC NONE
        button.min_width = None
        button.min_height = None

        assert button.min_width is None
        assert button.min_height is None

    def test_loader_button_mouse_press_event(self, qt_widget_cleanup):
        """Test de l'événement mousePressEvent."""
        button = LoaderButton()

        # ////// CRÉER UN VRAI ÉVÉNEMENT MOUSE QT
        from PySide6.QtCore import QPoint
        from PySide6.QtGui import QMouseEvent

        # Créer un vrai événement mouse press
        event = QMouseEvent(
            QMouseEvent.Type.MouseButtonPress,
            QPoint(10, 10),
            QPoint(10, 10),
            Qt.LeftButton,
            Qt.LeftButton,
            Qt.NoModifier,
        )

        # ////// TESTER QUE L'ÉVÉNEMENT NE LÈVE PAS D'EXCEPTION
        try:
            button.mousePressEvent(event)
        except Exception as e:
            pytest.fail(f"mousePressEvent() a levé une exception: {e}")

    def test_loader_button_mouse_press_event_loading(self, qt_widget_cleanup):
        """Test de mousePressEvent pendant le chargement."""
        button = LoaderButton()

        # ////// DÉMARRER LE CHARGEMENT
        button.start_loading()

        # ////// CRÉER UN VRAI ÉVÉNEMENT MOUSE QT
        from PySide6.QtCore import QPoint
        from PySide6.QtGui import QMouseEvent

        # Créer un vrai événement mouse press
        event = QMouseEvent(
            QMouseEvent.Type.MouseButtonPress,
            QPoint(10, 10),
            QPoint(10, 10),
            Qt.LeftButton,
            Qt.LeftButton,
            Qt.NoModifier,
        )

        # ////// TESTER QUE L'ÉVÉNEMENT NE LÈVE PAS D'EXCEPTION
        try:
            button.mousePressEvent(event)
        except Exception as e:
            pytest.fail(f"mousePressEvent() a levé une exception: {e}")

    def test_loader_button_mouse_press_event_right_button(self, qt_widget_cleanup):
        """Test de mousePressEvent avec bouton droit (doit être ignoré)."""
        button = LoaderButton()

        # ////// CRÉER UN ÉVÉNEMENT MOUSE AVEC BOUTON DROIT
        from PySide6.QtCore import QPoint
        from PySide6.QtGui import QMouseEvent

        event = QMouseEvent(
            QMouseEvent.Type.MouseButtonPress,
            QPoint(10, 10),
            QPoint(10, 10),
            Qt.RightButton,
            Qt.RightButton,
            Qt.NoModifier,
        )

        # ////// TESTER QUE L'ÉVÉNEMENT NE LÈVE PAS D'EXCEPTION
        try:
            button.mousePressEvent(event)
        except Exception as e:
            pytest.fail(f"mousePressEvent() a levé une exception: {e}")

    def test_loader_button_animation_speed(self, qt_widget_cleanup):
        """Test de la vitesse d'animation."""
        button = LoaderButton(animation_speed=50)

        # ////// VÉRIFIER LA VITESSE INITIALE
        assert button.animation_speed == 50

        # ////// MODIFIER LA VITESSE
        button.animation_speed = 75
        assert button.animation_speed == 75

    def test_loader_button_display_times(self, qt_widget_cleanup):
        """Test des temps d'affichage."""
        button = LoaderButton(success_display_time=1500, error_display_time=2500)

        # ////// VÉRIFIER LES TEMPS INITIAUX
        assert button.success_display_time == 1500
        assert button.error_display_time == 2500

        # ////// MODIFIER LES TEMPS
        button.success_display_time = 2000
        button.error_display_time = 3000

        assert button.success_display_time == 2000
        assert button.error_display_time == 3000

    @patch("ezqt_widgets.button.loader_button.QTimer")
    def test_loader_button_timer_integration(self, mock_timer_class, qt_widget_cleanup):
        """Test de l'intégration avec QTimer."""
        button = LoaderButton()

        # ////// VÉRIFIER QUE LES TIMERS SONT CRÉÉS
        # Note: Les timers sont créés dans _setup_animations
        assert mock_timer_class.call_count >= 0  # Au moins 0 timers créés

    def test_loader_button_state_transitions(self, qt_widget_cleanup):
        """Test des transitions d'état."""
        button = LoaderButton()

        # ////// ÉTAT INITIAL
        assert not button.is_loading

        # ////// TRANSITION VERS CHARGEMENT
        button.start_loading()
        assert button.is_loading

        # ////// TRANSITION VERS SUCCÈS
        button.stop_loading(success=True)
        assert not button.is_loading

        # ////// TRANSITION VERS ERREUR
        button.start_loading()
        button.stop_loading(success=False, error_message="Error")
        assert not button.is_loading



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
Tests unitaires pour le widget IndicatorLabel.
"""

import pytest
from PySide6.QtCore import QSize, Qt
from PySide6.QtWidgets import QApplication

from ezqt_widgets.label.indicator_label import IndicatorLabel


pytestmark = pytest.mark.unit


class TestIndicatorLabel:
    """Tests pour la classe IndicatorLabel."""

    def test_indicator_label_creation_default(self, qt_widget_cleanup):
        """Test de création avec paramètres par défaut."""
        label = IndicatorLabel()

        assert label is not None
        assert isinstance(label, IndicatorLabel)
        assert label.status == "neutral"

    def test_indicator_label_creation_with_parameters(self, qt_widget_cleanup):
        """Test de création avec paramètres personnalisés."""
        custom_status_map = {
            "custom1": {"text": "Custom 1", "state": "state1", "color": "#FF0000"},
            "custom2": {"text": "Custom 2", "state": "state2", "color": "#00FF00"},
        }

        label = IndicatorLabel(status_map=custom_status_map, initial_status="custom1")

        assert label.status == "custom1"

    def test_indicator_label_properties(self, qt_widget_cleanup):
        """Test des propriétés du label."""
        label = IndicatorLabel()

        # ////// TEST STATUS PROPERTY
        label.status = "online"
        assert label.status == "online"

        # ////// TESTER AVEC UN STATUT INVALIDE (DOIT LÉVER UNE EXCEPTION)
        with pytest.raises(ValueError, match="Unknown status"):
            label.status = "invalid_status"

    def test_indicator_label_signals(self, qt_widget_cleanup):
        """Test des signaux du label."""
        label = IndicatorLabel()

        # ////// TEST STATUSCHANGED SIGNAL
        signal_received = False
        received_status = ""

        def on_status_changed(status):
            nonlocal signal_received, received_status
            signal_received = True
            received_status = status

        label.statusChanged.connect(on_status_changed)

        # ////// CHANGER LE STATUT
        label.status = "online"

        # ////// VÉRIFIER QUE LE SIGNAL A ÉTÉ ÉMIS
        assert signal_received
        assert received_status == "online"

    def test_indicator_label_set_status_method(self, qt_widget_cleanup):
        """Test de la méthode set_status."""
        label = IndicatorLabel()

        # ////// ÉTAT INITIAL
        assert label.status == "neutral"

        # ////// CHANGER LE STATUT
        label.set_status("online")
        assert label.status == "online"

        # ////// CHANGER ENCORE
        label.set_status("offline")
        assert label.status == "offline"

    def test_indicator_label_refresh_style(self, qt_widget_cleanup):
        """Test de la méthode refresh_style."""
        label = IndicatorLabel()

        # ////// LA MÉTHODE NE DOIT PAS LEVER D'EXCEPTION
        try:
            label.refresh_style()
        except Exception as e:
            pytest.fail(f"refresh_style() a levé une exception: {e}")

    def test_indicator_label_default_status_map(self, qt_widget_cleanup):
        """Test de la carte de statuts par défaut."""
        label = IndicatorLabel()

        # ////// VÉRIFIER LES STATUTS PAR DÉFAUT
        assert label.status == "neutral"

        # ////// TESTER LES DIFFÉRENTS STATUTS
        label.status = "online"
        assert label.status == "online"

        label.status = "offline"
        assert label.status == "offline"

        # ////// TESTER LES STATUTS QUI N'EXISTENT PAS
        with pytest.raises(ValueError, match="Unknown status"):
            label.status = "error"

        with pytest.raises(ValueError, match="Unknown status"):
            label.status = "warning"

        with pytest.raises(ValueError, match="Unknown status"):
            label.status = "success"

    def test_indicator_label_custom_status_map(self, qt_widget_cleanup):
        """Test avec une carte de statuts personnalisée."""
        custom_map = {
            "ready": {"text": "Prêt", "state": "ready", "color": "#4CAF50"},
            "busy": {"text": "Occupé", "state": "busy", "color": "#FF9800"},
            "error": {"text": "Erreur", "state": "error", "color": "#F44336"},
        }

        label = IndicatorLabel(status_map=custom_map, initial_status="ready")

        # ////// VÉRIFIER LE STATUT INITIAL
        assert label.status == "ready"

        # ////// TESTER LES AUTRES STATUTS
        label.status = "busy"
        assert label.status == "busy"

        label.status = "error"
        assert label.status == "error"

        # ////// TESTER UN STATUT QUI N'EXISTE PAS
        with pytest.raises(ValueError, match="Unknown status"):
            label.status = "unknown"

    def test_indicator_label_status_transitions(self, qt_widget_cleanup):
        """Test des transitions de statut."""
        label = IndicatorLabel()

        # ////// ÉTAT INITIAL
        assert label.status == "neutral"

        # ////// TRANSITION VERS ONLINE
        label.status = "online"
        assert label.status == "online"

        # ////// TRANSITION VERS OFFLINE
        label.status = "offline"
        assert label.status == "offline"

        # ////// TRANSITION VERS NEUTRAL
        label.status = "neutral"
        assert label.status == "neutral"

    def test_indicator_label_property_type(self, qt_widget_cleanup):
        """Test de la propriété type pour QSS."""
        label = IndicatorLabel()

        # ////// VÉRIFIER QUE LA PROPRIÉTÉ TYPE EST DÉFINIE
        assert label.property("type") == "IndicatorLabel"

    def test_indicator_label_multiple_instances(self, qt_widget_cleanup):
        """Test de plusieurs instances."""
        # ////// CRÉER PLUSIEURS INSTANCES
        label1 = IndicatorLabel(initial_status="online")
        label2 = IndicatorLabel(initial_status="offline")
        label3 = IndicatorLabel(initial_status="neutral")

        # ////// VÉRIFIER QUE CHAQUE INSTANCE EST INDÉPENDANTE
        assert label1.status == "online"
        assert label2.status == "offline"
        assert label3.status == "neutral"

        # ////// MODIFIER UNE INSTANCE
        label1.status = "offline"
        assert label1.status == "offline"
        assert label2.status == "offline"  # Non affecté
        assert label3.status == "neutral"  # Non affecté

    def test_indicator_label_empty_status_map(self, qt_widget_cleanup):
        """Test avec une carte de statuts vide."""
        empty_map = {}
        label = IndicatorLabel(status_map=empty_map)

        # ////// VÉRIFIER QUE LE WIDGET EST CRÉÉ
        assert label is not None
        assert isinstance(label, IndicatorLabel)

        # ////// TESTER UN CHANGEMENT DE STATUT (DOIT LÉVER UNE EXCEPTION)
        with pytest.raises(ValueError, match="Unknown status"):
            label.status = "any_status"

    def test_indicator_label_invalid_initial_status(self, qt_widget_cleanup):
        """Test avec un statut initial invalide."""
        # ////// CRÉER UN LABEL AVEC UN STATUT INVALIDE (DOIT LÉVER UNE EXCEPTION)
        with pytest.raises(ValueError, match="Unknown status"):
            IndicatorLabel(initial_status="invalid_status")

    def test_indicator_label_status_map_structure(self, qt_widget_cleanup):
        """Test de la structure de la carte de statuts."""
        # ////// CARTE AVEC STRUCTURE COMPLÈTE
        complete_map = {
            "test1": {"text": "Test 1", "state": "state1", "color": "#FF0000"},
            "test2": {"text": "Test 2", "state": "state2", "color": "#00FF00"},
        }

        label = IndicatorLabel(status_map=complete_map, initial_status="test1")
        assert label.status == "test1"

        # ////// CARTE AVEC STRUCTURE INCOMPLÈTE
        incomplete_map = {
            "test3": {
                "text": "Test 3"
                # Manque "state" et "color"
            }
        }

        label2 = IndicatorLabel(status_map=incomplete_map, initial_status="test3")
        assert label2.status == "test3"

    def test_indicator_label_status_changes_with_signals(self, qt_widget_cleanup):
        """Test des changements de statut avec signaux."""
        label = IndicatorLabel()

        # ////// CONNECTER LE SIGNAL
        signal_count = 0
        received_statuses = []

        def on_status_changed(status):
            nonlocal signal_count, received_statuses
            signal_count += 1
            received_statuses.append(status)

        label.statusChanged.connect(on_status_changed)

        # ////// CHANGER LE STATUT PLUSIEURS FOIS
        label.status = "online"
        label.status = "offline"
        label.status = "neutral"

        # ////// VÉRIFIER LES SIGNALS
        assert signal_count == 3
        assert received_statuses == ["online", "offline", "neutral"]

    def test_indicator_label_same_status_multiple_times(self, qt_widget_cleanup):
        """Test du même statut défini plusieurs fois."""
        label = IndicatorLabel()

        # ////// CONNECTER LE SIGNAL
        signal_count = 0

        def on_status_changed(status):
            nonlocal signal_count
            signal_count += 1

        label.statusChanged.connect(on_status_changed)

        # ////// DÉFINIR LE MÊME STATUT PLUSIEURS FOIS
        label.status = "online"
        label.status = "online"
        label.status = "online"

        # ////// VÉRIFIER QUE LE SIGNAL EST ÉMIS À CHAQUE FOIS
        # Note: Certains widgets peuvent ne pas émettre le signal si la valeur ne change pas
        # Vérifions plutôt que le statut final est correct
        assert label.status == "online"
        # Le signal peut être émis 1 fois (première définition) ou 3 fois selon l'implémentation
        assert signal_count >= 1

    def test_indicator_label_constructor_without_parameters(self, qt_widget_cleanup):
        """Test du constructeur sans paramètres."""
        label = IndicatorLabel()

        # ////// VÉRIFIER LES VALEURS PAR DÉFAUT
        assert label.status == "neutral"

        # ////// VÉRIFIER QUE LE WIDGET EST FONCTIONNEL
        label.status = "online"
        assert label.status == "online"



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
EzQt_Widgets package initialization.
"""
__version__ = "2.2.1"

# BUTTON WIDGETS
from .button import *

# INPUT WIDGETS
from .input import *

# LABEL WIDGETS
from .label import *

# MISC WIDGETS
from .misc import *



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# TAB REPLACE TEXTEDIT
from .tab_replace_textedit import *

# AUTO COMPLETE INPUT
from .auto_complete_input import *

# SEARCH INPUT
from .search_input import *

# PASSWORD INPUT
from .password_input import *


"""
Example runner module for EzQt Widgets CLI.

Handles the execution of example files with proper error handling and feedback.
"""

import os
import subprocess
import sys
from pathlib import Path
from typing import Optional, List
import click


class ExampleRunner:
    """Handles running EzQt Widgets examples."""

    def __init__(self, verbose: bool = False):
        self.verbose = verbose
        self.examples_dir = self._find_examples_dir()

    def _find_examples_dir(self) -> Path:
        """Find the examples directory relative to the package."""
        # Try to find examples in the project root
        package_dir = Path(__file__).parent.parent.parent
        examples_dir = package_dir / "examples"

        if examples_dir.exists():
            return examples_dir

        # Fallback: try to find examples in the current directory
        current_examples = Path.cwd() / "examples"
        if current_examples.exists():
            return current_examples

        # Last resort: try to find examples in the package
        package_examples = Path(__file__).parent.parent / "examples"
        if package_examples.exists():
            return package_examples

        raise FileNotFoundError("Examples directory not found")

    def get_available_examples(self) -> List[Path]:
        """Get list of available example files."""
        examples = []
        for pattern in ["*_example.py", "run_all_examples.py"]:
            examples.extend(self.examples_dir.glob(pattern))
        return sorted(examples)

    def run_example(self, example_name: str) -> bool:
        """Run a specific example by name."""
        example_path = self.examples_dir / f"{example_name}.py"

        if not example_path.exists():
            click.echo(f"❌ Example not found: {example_name}")
            return False

        return self._execute_example(example_path)

    def run_all_examples(self, use_gui_launcher: bool = True) -> bool:
        """Run all examples or use the GUI launcher."""
        if use_gui_launcher:
            launcher_path = self.examples_dir / "run_all_examples.py"
            if launcher_path.exists():
                return self._execute_example(launcher_path)
            else:
                click.echo("⚠️  GUI launcher not found, running examples sequentially")
                use_gui_launcher = False

        if not use_gui_launcher:
            # Run each example sequentially
            examples = [
                "button_example",
                "input_example",
                "label_example",
                "misc_example",
            ]
            success_count = 0

            for example in examples:
                click.echo(f"\n{'='*50}")
                click.echo(f"🚀 Running: {example}")
                click.echo(f"{'='*50}")

                if self.run_example(example):
                    success_count += 1
                else:
                    click.echo(f"❌ Failed to run: {example}")

            click.echo(
                f"\n✅ Successfully ran {success_count}/{len(examples)} examples"
            )
            return success_count == len(examples)

        return False

    def _execute_example(self, example_path: Path) -> bool:
        """Execute a specific example file."""
        if self.verbose:
            click.echo(f"🚀 Running: {example_path.name}")
            click.echo(f"📁 Path: {example_path}")

        try:
            # Change to examples directory before running
            examples_dir = example_path.parent
            original_cwd = os.getcwd()

            if self.verbose:
                click.echo(f"📂 Changing to directory: {examples_dir}")

            os.chdir(examples_dir)

            # Run the example
            result = subprocess.run(
                [sys.executable, str(example_path)],
                check=True,
                capture_output=not self.verbose,
            )

            if self.verbose:
                click.echo(f"✅ Successfully completed: {example_path.name}")

            return True

        except subprocess.CalledProcessError as e:
            click.echo(f"❌ Error running {example_path.name}: {e}")
            if self.verbose and e.stderr:
                click.echo(f"Error details: {e.stderr.decode()}")
            return False

        except KeyboardInterrupt:
            click.echo(f"\n⏹️  {example_path.name} stopped by user")
            return False

        except Exception as e:
            click.echo(f"❌ Unexpected error running {example_path.name}: {e}")
            return False
        finally:
            # Restore original working directory
            try:
                os.chdir(original_cwd)
            except:
                pass

    def list_examples(self) -> None:
        """List all available examples."""
        examples = self.get_available_examples()

        if not examples:
            click.echo("❌ No examples found")
            return

        click.echo("📋 Available examples:")
        click.echo("=" * 40)

        for example in examples:
            status = "✅" if example.exists() else "❌"
            click.echo(f"{status} {example.stem}")

        click.echo(f"\nTotal: {len(examples)} examples found")


def run_example_by_category(category: str, verbose: bool = False) -> bool:
    """Run examples by category."""
    runner = ExampleRunner(verbose)

    category_mapping = {
        "buttons": "button_example",
        "inputs": "input_example",
        "labels": "label_example",
        "misc": "misc_example",
    }

    if category not in category_mapping:
        click.echo(f"❌ Unknown category: {category}")
        click.echo(f"Available categories: {', '.join(category_mapping.keys())}")
        return False

    return runner.run_example(category_mapping[category])


def run_all_examples(use_gui: bool = True, verbose: bool = False) -> bool:
    """Run all examples."""
    runner = ExampleRunner(verbose)
    return runner.run_all_examples(use_gui)


def list_available_examples() -> None:
    """List all available examples."""
    runner = ExampleRunner()
    runner.list_examples()



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Qt,
    QStringListModel,
)
from PySide6.QtWidgets import (
    QLineEdit,
    QCompleter,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////

# ////// TYPE HINTS IMPROVEMENTS FOR PYSIDE6 6.9.1
from typing import Optional, List

# UTILITY FUNCTIONS
# ///////////////////////////////////////////////////////////////

# CLASS
# ///////////////////////////////////////////////////////////////


class AutoCompleteInput(QLineEdit):
    """
    QLineEdit subclass with autocompletion support.
    You can provide a list of suggestions (strings) to be used for autocompletion.

    Parameters
    ----------
    parent : QWidget, optional
        The parent widget (default: None).
    suggestions : List[str], optional
        List of strings to use for autocompletion (default: empty list).
    case_sensitive : bool, optional
        Whether the autocompletion is case sensitive (default: False).
    filter_mode : Qt.MatchFlag, optional
        Filter mode for completion (default: Qt.MatchContains).
    completion_mode : QCompleter.CompletionMode, optional
        Completion mode (default: QCompleter.PopupCompletion).
    *args, **kwargs :
        Additional arguments passed to QLineEdit.

    Properties
    ----------
    suggestions : List[str]
        Get or set the list of suggestions for autocompletion.
    case_sensitive : bool
        Get or set whether autocompletion is case sensitive.
    filter_mode : Qt.MatchFlag
        Get or set the filter mode for completion.
    completion_mode : QCompleter.CompletionMode
        Get or set the completion mode.
    """

    def __init__(
        self,
        parent=None,
        suggestions: Optional[List[str]] = None,
        case_sensitive: bool = False,
        filter_mode: Qt.MatchFlag = Qt.MatchContains,
        completion_mode: QCompleter.CompletionMode = QCompleter.PopupCompletion,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(parent, *args, **kwargs)

        # ////// INITIALIZE PROPERTIES
        self._suggestions: List[str] = suggestions or []
        self._case_sensitive: bool = case_sensitive
        self._filter_mode: Qt.MatchFlag = filter_mode
        self._completion_mode: QCompleter.CompletionMode = completion_mode

        # ////// SETUP COMPLETER
        self._setup_completer()

    def _setup_completer(self) -> None:
        """Setup the completer with current settings."""
        self._completer = QCompleter(self)
        self._model = QStringListModel(self._suggestions, self)

        # ////// CONFIGURE COMPLETER
        self._completer.setModel(self._model)
        self._completer.setCaseSensitivity(
            Qt.CaseSensitive if self._case_sensitive else Qt.CaseInsensitive
        )
        self._completer.setFilterMode(self._filter_mode)
        self._completer.setCompletionMode(self._completion_mode)
        self.setCompleter(self._completer)

    # PROPERTIES
    # ///////////////////////////////////////////////////////////////

    @property
    def suggestions(self) -> List[str]:
        """Get the list of suggestions."""
        return self._suggestions.copy()

    @suggestions.setter
    def suggestions(self, value: List[str]) -> None:
        """Set the list of suggestions."""
        self._suggestions = value or []
        self._model.setStringList(self._suggestions)

    @property
    def case_sensitive(self) -> bool:
        """Get whether autocompletion is case sensitive."""
        return self._case_sensitive

    @case_sensitive.setter
    def case_sensitive(self, value: bool) -> None:
        """Set whether autocompletion is case sensitive."""
        self._case_sensitive = bool(value)
        self._completer.setCaseSensitivity(
            Qt.CaseSensitive if self._case_sensitive else Qt.CaseInsensitive
        )

    @property
    def filter_mode(self) -> Qt.MatchFlag:
        """Get the filter mode for completion."""
        return self._filter_mode

    @filter_mode.setter
    def filter_mode(self, value: Qt.MatchFlag) -> None:
        """Set the filter mode for completion."""
        self._filter_mode = value
        self._completer.setFilterMode(self._filter_mode)

    @property
    def completion_mode(self) -> QCompleter.CompletionMode:
        """Get the completion mode."""
        return self._completion_mode

    @completion_mode.setter
    def completion_mode(self, value: QCompleter.CompletionMode) -> None:
        """Set the completion mode."""
        self._completion_mode = value
        self._completer.setCompletionMode(self._completion_mode)

    # UTILITY FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def add_suggestion(self, suggestion: str) -> None:
        """Add a suggestion to the list."""
        if suggestion and suggestion not in self._suggestions:
            self._suggestions.append(suggestion)
            self._model.setStringList(self._suggestions)

    def remove_suggestion(self, suggestion: str) -> None:
        """Remove a suggestion from the list."""
        if suggestion in self._suggestions:
            self._suggestions.remove(suggestion)
            self._model.setStringList(self._suggestions)

    def clear_suggestions(self) -> None:
        """Clear all suggestions."""
        self._suggestions.clear()
        self._model.setStringList(self._suggestions)

    # STYLE FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def refresh_style(self) -> None:
        """Refresh the widget's style (useful after dynamic stylesheet changes)."""
        self.style().unpolish(self)
        self.style().polish(self)
        self.update()



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# DATE BUTTON
from .date_button import *

# ICON BUTTON
from .icon_button import *

# LOADER BUTTON
from .loader_button import *


# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
Tests unitaires pour le widget FramedLabel.
"""

import pytest
from PySide6.QtCore import QSize, Qt
from PySide6.QtWidgets import QApplication

from ezqt_widgets.label.framed_label import FramedLabel


pytestmark = pytest.mark.unit


class TestFramedLabel:
    """Tests pour la classe FramedLabel."""

    def test_framed_label_creation_default(self, qt_widget_cleanup):
        """Test de création avec paramètres par défaut."""
        label = FramedLabel()

        assert label is not None
        assert isinstance(label, FramedLabel)
        assert label.text == ""
        assert label.alignment == Qt.AlignmentFlag.AlignCenter
        assert label.min_width is None
        assert label.min_height is None

    def test_framed_label_creation_with_parameters(self, qt_widget_cleanup):
        """Test de création avec paramètres personnalisés."""
        label = FramedLabel(
            text="Test Label",
            alignment=Qt.AlignmentFlag.AlignLeft,
            style_sheet="background-color: red;",
            min_width=200,
            min_height=50,
        )

        assert label.text == "Test Label"
        assert label.alignment == Qt.AlignmentFlag.AlignLeft
        assert label.min_width == 200
        assert label.min_height == 50

    def test_framed_label_properties(self, qt_widget_cleanup):
        """Test des propriétés du label."""
        label = FramedLabel()

        # ////// TEST TEXT PROPERTY
        label.text = "New Text"
        assert label.text == "New Text"

        # ////// TEST ALIGNMENT PROPERTY
        label.alignment = Qt.AlignmentFlag.AlignRight
        assert label.alignment == Qt.AlignmentFlag.AlignRight

        # ////// TEST MIN_WIDTH PROPERTY
        label.min_width = 150
        assert label.min_width == 150

        # ////// TEST MIN_HEIGHT PROPERTY
        label.min_height = 40
        assert label.min_height == 40

        # ////// TESTER AVEC NONE
        label.min_width = None
        label.min_height = None
        assert label.min_width is None
        assert label.min_height is None

    def test_framed_label_signals(self, qt_widget_cleanup):
        """Test des signaux du label."""
        label = FramedLabel()

        # ////// TEST TEXTCHANGED SIGNAL
        signal_received = False
        received_text = ""

        def on_text_changed(text):
            nonlocal signal_received, received_text
            signal_received = True
            received_text = text

        label.textChanged.connect(on_text_changed)

        # ////// CHANGER LE TEXTE
        label.text = "Signal Test"

        # ////// VÉRIFIER QUE LE SIGNAL A ÉTÉ ÉMIS
        assert signal_received
        assert received_text == "Signal Test"

    def test_framed_label_size_hints(self, qt_widget_cleanup):
        """Test des méthodes de taille."""
        label = FramedLabel(text="Test Label")

        # ////// TEST MINIMUMSIZEHINT
        min_size_hint = label.minimumSizeHint()
        assert min_size_hint is not None
        assert isinstance(min_size_hint, QSize)
        assert min_size_hint.width() > 0
        assert min_size_hint.height() > 0

    def test_framed_label_refresh_style(self, qt_widget_cleanup):
        """Test de la méthode refresh_style."""
        label = FramedLabel()

        # ////// LA MÉTHODE NE DOIT PAS LEVER D'EXCEPTION
        try:
            label.refresh_style()
        except Exception as e:
            pytest.fail(f"refresh_style() a levé une exception: {e}")

    def test_framed_label_alignment_options(self, qt_widget_cleanup):
        """Test des différentes options d'alignement."""
        # ////// TEST ALIGNMENT LEFT
        label_left = FramedLabel(alignment=Qt.AlignmentFlag.AlignLeft)
        assert label_left.alignment == Qt.AlignmentFlag.AlignLeft

        # ////// TEST ALIGNMENT CENTER
        label_center = FramedLabel(alignment=Qt.AlignmentFlag.AlignCenter)
        assert label_center.alignment == Qt.AlignmentFlag.AlignCenter

        # ////// TEST ALIGNMENT RIGHT
        label_right = FramedLabel(alignment=Qt.AlignmentFlag.AlignRight)
        assert label_right.alignment == Qt.AlignmentFlag.AlignRight

        # ////// TEST ALIGNMENT TOP
        label_top = FramedLabel(alignment=Qt.AlignmentFlag.AlignTop)
        assert label_top.alignment == Qt.AlignmentFlag.AlignTop

        # ////// TEST ALIGNMENT BOTTOM
        label_bottom = FramedLabel(alignment=Qt.AlignmentFlag.AlignBottom)
        assert label_bottom.alignment == Qt.AlignmentFlag.AlignBottom

    def test_framed_label_text_changes(self, qt_widget_cleanup):
        """Test des changements de texte."""
        label = FramedLabel()

        # ////// TEXTE VIDE
        label.text = ""
        assert label.text == ""

        # ////// TEXTE AVEC ESPACES
        label.text = "   Text with spaces   "
        assert label.text == "   Text with spaces   "

        # ////// TEXTE LONG
        long_text = "This is a very long text that should be handled properly by the FramedLabel widget"
        label.text = long_text
        assert label.text == long_text

        # ////// TEXTE AVEC CARACTÈRES SPÉCIAUX
        special_text = "Text with special chars: éàùç€£¥"
        label.text = special_text
        assert label.text == special_text

    def test_framed_label_style_sheet(self, qt_widget_cleanup):
        """Test de l'application de stylesheet."""
        # ////// CRÉER UN LABEL AVEC STYLESHEET
        style_sheet = "background-color: #FF0000; color: white; border: 2px solid blue;"
        label = FramedLabel(style_sheet=style_sheet)

        # ////// VÉRIFIER QUE LE STYLESHEET EST APPLIQUÉ
        # Note: On ne peut pas facilement tester le rendu visuel dans les tests unitaires
        # mais on peut vérifier que le widget est créé sans erreur
        assert label is not None
        assert isinstance(label, FramedLabel)

    def test_framed_label_dimensions(self, qt_widget_cleanup):
        """Test des dimensions minimales."""
        label = FramedLabel(min_width=100, min_height=30)

        # ////// VÉRIFIER LES DIMENSIONS INITIALES
        assert label.min_width == 100
        assert label.min_height == 30

        # ////// MODIFIER LES DIMENSIONS
        label.min_width = 200
        label.min_height = 50
        assert label.min_width == 200
        assert label.min_height == 50

        # ////// TESTER AVEC DES VALEURS NÉGATIVES
        label.min_width = -10
        label.min_height = -5
        assert label.min_width == -10
        assert label.min_height == -5

    def test_framed_label_property_type(self, qt_widget_cleanup):
        """Test de la propriété type pour QSS."""
        label = FramedLabel()

        # ////// VÉRIFIER QUE LA PROPRIÉTÉ TYPE EST DÉFINIE
        assert label.property("type") == "FramedLabel"

    def test_framed_label_multiple_instances(self, qt_widget_cleanup):
        """Test de plusieurs instances."""
        # ////// CRÉER PLUSIEURS INSTANCES
        label1 = FramedLabel(text="Label 1")
        label2 = FramedLabel(text="Label 2")
        label3 = FramedLabel(text="Label 3")

        # ////// VÉRIFIER QUE CHAQUE INSTANCE EST INDÉPENDANTE
        assert label1.text == "Label 1"
        assert label2.text == "Label 2"
        assert label3.text == "Label 3"

        # ////// MODIFIER UNE INSTANCE
        label1.text = "Modified Label 1"
        assert label1.text == "Modified Label 1"
        assert label2.text == "Label 2"  # Non affecté
        assert label3.text == "Label 3"  # Non affecté

    def test_framed_label_empty_constructor(self, qt_widget_cleanup):
        """Test du constructeur sans paramètres."""
        label = FramedLabel()

        # ////// VÉRIFIER LES VALEURS PAR DÉFAUT
        assert label.text == ""
        assert label.alignment == Qt.AlignmentFlag.AlignCenter
        assert label.min_width is None
        assert label.min_height is None

    def test_framed_label_text_property_changes(self, qt_widget_cleanup):
        """Test des changements de la propriété text."""
        label = FramedLabel()

        # ////// CHANGER LE TEXTE PLUSIEURS FOIS
        label.text = "First text"
        assert label.text == "First text"

        label.text = "Second text"
        assert label.text == "Second text"

        label.text = "Third text"
        assert label.text == "Third text"

        # ////// REVENIR À UN TEXTE VIDE
        label.text = ""
        assert label.text == ""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
Exemple d'utilisation des widgets de boutons EzQt_Widgets.

Ce script démontre l'utilisation de tous les types de boutons disponibles :
- DateButton
- IconButton  
- LoaderButton
"""

import sys
import os
import re
import yaml
from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QFont
from PySide6.QtWidgets import (
    QApplication,
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QLabel,
    QPushButton,
    QGroupBox,
    QMessageBox,
    QScrollArea,
)

# Import des widgets EzQt
from ezqt_widgets.button import DateButton, IconButton, LoaderButton


def load_and_apply_qss(app, qss_path, yaml_path, theme_name="dark"):
    """Fonction utilitaire pour charger et appliquer le QSS avec variables."""
    try:
        # Charger les variables du thème depuis le YAML
        with open(yaml_path, encoding="utf-8") as f:
            config = yaml.safe_load(f)
        theme_vars = config["theme_palette"][theme_name]
        
        # Charger le QSS
        with open(qss_path, encoding="utf-8") as f:
            qss = f.read()
        
        # Remplacer les variables $ par leur valeur
        def repl(match):
            var = match.group(0)
            return theme_vars.get(var, var)
        
        qss = re.sub(r"\$_[a-zA-Z0-9_]+", repl, qss)
        app.setStyleSheet(qss)
        print(f"Thème '{theme_name}' chargé avec succès")
        
    except Exception as e:
        print(f"Erreur lors du chargement du thème: {e}")
        # Appliquer un style par défaut en cas d'erreur
        app.setStyleSheet("")


class ButtonExampleWidget(QWidget):
    """Widget de démonstration pour tous les types de boutons."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Exemples de Boutons - EzQt_Widgets")
        self.setMinimumSize(800, 600)
        self.setup_ui()

    def setup_ui(self):
        """Configure l'interface utilisateur."""
        # Layout principal
        main_layout = QVBoxLayout(self)
        main_layout.setSpacing(0)
        main_layout.setContentsMargins(0, 0, 0, 0)

        # Création du ScrollArea
        scroll_area = QScrollArea()
        scroll_area.setWidgetResizable(True)
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        scroll_area.setFrameShape(QScrollArea.NoFrame)

        # Widget conteneur pour le contenu
        content_widget = QWidget()
        content_layout = QVBoxLayout(content_widget)
        content_layout.setSpacing(20)
        content_layout.setContentsMargins(20, 20, 20, 20)

        # Titre
        title = QLabel("Exemples de Boutons - EzQt_Widgets")
        title.setFont(QFont("Segoe UI", 16, QFont.Bold))
        title.setAlignment(Qt.AlignCenter)
        title.setStyleSheet("color: #2c3e50; margin-bottom: 20px;")
        content_layout.addWidget(title)

        # Groupe DateButton
        date_group = QGroupBox("DateButton")
        date_layout = QVBoxLayout(date_group)

        date_label = QLabel("Sélecteur de date avec validation:")
        date_layout.addWidget(date_label)

        self.date_button = DateButton()
        self.date_button.date = "2024-01-15"
        self.date_button.dateChanged.connect(self.on_date_changed)
        date_layout.addWidget(self.date_button)

        self.date_output = QLabel("Date sélectionnée: 2024-01-15")
        self.date_output.setStyleSheet("font-weight: bold; color: #4CAF50;")
        date_layout.addWidget(self.date_output)

        content_layout.addWidget(date_group)

        # Groupe IconButton
        icon_group = QGroupBox("IconButton")
        icon_layout = QVBoxLayout(icon_group)

        icon_label = QLabel("Bouton avec icône et texte:")
        icon_layout.addWidget(icon_label)

        # Création d'une icône simple pour l'exemple
        from PySide6.QtGui import QPixmap, QIcon

        pixmap = QPixmap(32, 32)
        pixmap.fill(Qt.blue)
        icon = QIcon(pixmap)

        self.icon_button = IconButton()
        self.icon_button.setIcon(icon)
        self.icon_button.setText("Bouton avec Icône")
        self.icon_button.clicked.connect(self.on_icon_button_clicked)
        icon_layout.addWidget(self.icon_button)

        self.icon_output = QLabel("Clics: 0")
        self.icon_output.setStyleSheet("font-weight: bold; color: #2196F3;")
        icon_layout.addWidget(self.icon_output)

        content_layout.addWidget(icon_group)

        # Groupe LoaderButton
        loader_group = QGroupBox("LoaderButton")
        loader_layout = QVBoxLayout(loader_group)

        loader_label = QLabel("Bouton de chargement avec états:")
        loader_layout.addWidget(loader_label)

        # Création du LoaderButton avec configuration complète
        self.loader_button = LoaderButton(
            text="Charger des données",
            loading_text="Chargement en cours...",
            auto_reset=True,
            success_display_time=2000,
            error_display_time=3000,
        )
        self.loader_button.clicked.connect(self.on_loader_button_clicked)
        loader_layout.addWidget(self.loader_button)

        self.loader_output = QLabel("État: Prêt")
        self.loader_output.setStyleSheet("font-weight: bold; color: #FF9800;")
        loader_layout.addWidget(self.loader_output)

        content_layout.addWidget(loader_group)

        # Boutons de test
        test_group = QGroupBox("Tests Interactifs")
        test_layout = QHBoxLayout(test_group)

        test_date_btn = QPushButton("Test Date")
        test_date_btn.clicked.connect(self.test_date_button)
        test_layout.addWidget(test_date_btn)

        test_loader_btn = QPushButton("Test Loader")
        test_loader_btn.clicked.connect(self.test_loader_button)
        test_layout.addWidget(test_loader_btn)

        reset_btn = QPushButton("Reset")
        reset_btn.clicked.connect(self.reset_all)
        test_layout.addWidget(reset_btn)

        content_layout.addWidget(test_group)

        # Espacement en bas pour éviter que le contenu soit coupé
        content_layout.addStretch()

        # Configuration du ScrollArea
        scroll_area.setWidget(content_widget)
        main_layout.addWidget(scroll_area)

        # Compteur de clics pour IconButton
        self.icon_click_count = 0

    def on_date_changed(self, date):
        """Callback appelé quand la date change."""
        self.date_output.setText(f"Date sélectionnée: {date}")
        print(f"Date changée: {date}")

    def on_icon_button_clicked(self):
        """Callback appelé quand le bouton icône est cliqué."""
        self.icon_click_count += 1
        self.icon_output.setText(f"Clics: {self.icon_click_count}")
        print(f"Bouton icône cliqué! Total: {self.icon_click_count}")

    def on_loader_button_clicked(self):
        """Callback appelé quand le bouton de chargement est cliqué."""
        self.loader_output.setText("État: Chargement...")
        print("Démarrage du chargement...")

        # Simuler un chargement
        QTimer.singleShot(2000, self.simulate_loading_complete)

    def simulate_loading_complete(self):
        """Simule la fin du chargement."""
        import random

        success = random.choice([True, False])

        if success:
            self.loader_output.setText("État: Succès!")
            print("Chargement réussi!")
        else:
            self.loader_output.setText("État: Erreur!")
            print("Erreur de chargement!")

    def test_date_button(self):
        """Teste le bouton de date."""
        from datetime import datetime, timedelta

        new_date = datetime.now() + timedelta(days=7)
        self.date_button.date = new_date.strftime("%Y-%m-%d")
        print(f"Test: Date changée vers {new_date.strftime('%Y-%m-%d')}")

    def test_loader_button(self):
        """Teste le bouton de chargement."""
        self.loader_button.click()
        print("Test: Démarrage du chargement")

    def reset_all(self):
        """Remet tout à zéro."""
        self.icon_click_count = 0
        self.icon_output.setText("Clics: 0")
        self.loader_output.setText("État: Prêt")
        self.date_button.date = "2024-01-15"
        print("Reset: Tous les compteurs remis à zéro")


def main():
    """Fonction principale pour l'exécution individuelle."""
    app = QApplication(sys.argv)

    # Charger le thème depuis les fichiers
    bin_dir = os.path.join(os.path.dirname(__file__), "bin")
    qss_path = os.path.join(bin_dir, "main_theme.qss")
    yaml_path = os.path.join(bin_dir, "app.yaml")
    
    if os.path.exists(qss_path) and os.path.exists(yaml_path):
        load_and_apply_qss(app, qss_path, yaml_path, theme_name="dark")
    else:
        # Style par défaut si les fichiers de thème ne sont pas trouvés
        app.setStyleSheet("""
            QGroupBox {
                font-weight: bold;
                border: 2px solid #cccccc;
                border-radius: 5px;
                margin-top: 1ex;
                padding-top: 10px;
            }
            QPushButton {
                background-color: #4CAF50;
                border: none;
                color: white;
                padding: 8px 16px;
                text-align: center;
                font-size: 14px;
                border-radius: 4px;
            }
            QPushButton:hover {
                background-color: #45a049;
            }
            QLabel {
                color: #2c3e50;
            }
            QScrollArea {
                border: none;
                background-color: transparent;
            }
        """)

    window = ButtonExampleWidget()
    window.show()

    sys.exit(app.exec())


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
Tests unitaires pour le widget TabReplaceTextEdit (version corrigée).
"""

import pytest
from unittest.mock import MagicMock
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QApplication

from ezqt_widgets.input.tab_replace_textedit import TabReplaceTextEdit


pytestmark = pytest.mark.unit


class TestTabReplaceTextEdit:
    """Tests pour la classe TabReplaceTextEdit."""

    def test_tab_replace_textedit_creation_default(self, qt_widget_cleanup):
        """Test de création avec paramètres par défaut."""
        text_edit = TabReplaceTextEdit()

        assert text_edit is not None
        assert isinstance(text_edit, TabReplaceTextEdit)
        assert text_edit.tab_replacement == "\n"
        assert text_edit.sanitize_on_paste is True
        assert text_edit.remove_empty_lines is True
        assert text_edit.preserve_whitespace is False

    def test_tab_replace_textedit_creation_with_parameters(self, qt_widget_cleanup):
        """Test de création avec paramètres personnalisés."""
        text_edit = TabReplaceTextEdit(
            tab_replacement=" ",
            sanitize_on_paste=False,
            remove_empty_lines=False,
            preserve_whitespace=True,
        )

        assert text_edit.tab_replacement == " "
        assert text_edit.sanitize_on_paste is False
        assert text_edit.remove_empty_lines is False
        assert text_edit.preserve_whitespace is True

    def test_tab_replace_textedit_properties(self, qt_widget_cleanup):
        """Test des propriétés du widget."""
        text_edit = TabReplaceTextEdit()

        # ////// TEST TAB_REPLACEMENT PROPERTY
        text_edit.tab_replacement = "  "
        assert text_edit.tab_replacement == "  "

        # ////// TEST SANITIZE_ON_PASTE PROPERTY
        text_edit.sanitize_on_paste = False
        assert text_edit.sanitize_on_paste is False

        # ////// TEST REMOVE_EMPTY_LINES PROPERTY
        text_edit.remove_empty_lines = False
        assert text_edit.remove_empty_lines is False

        # ////// TEST PRESERVE_WHITESPACE PROPERTY
        text_edit.preserve_whitespace = True
        assert text_edit.preserve_whitespace is True

    def test_tab_replace_textedit_sanitize_text_basic(self, qt_widget_cleanup):
        """Test de la méthode sanitize_text avec des cas de base."""
        text_edit = TabReplaceTextEdit()

        # ////// TEST REMPLACEMENT DE TAB PAR DÉFAUT
        text = "col1\tcol2\tcol3"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "col1\ncol2\ncol3"

        # ////// TEST SANS TAB
        text = "no tabs here"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "no tabs here"

    def test_tab_replace_textedit_sanitize_text_custom_replacement(
        self, qt_widget_cleanup
    ):
        """Test de sanitize_text avec un remplacement personnalisé."""
        text_edit = TabReplaceTextEdit(tab_replacement="  ")

        # ////// TEST REMPLACEMENT PAR ESPACES
        text = "col1\tcol2\tcol3"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "col1  col2  col3"

        # ////// TEST REMPLACEMENT PAR VIRGULE
        text_edit.tab_replacement = ","
        text = "col1\tcol2\tcol3"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "col1,col2,col3"

    def test_tab_replace_textedit_sanitize_text_remove_empty_lines(
        self, qt_widget_cleanup
    ):
        """Test de sanitize_text avec suppression des lignes vides."""
        text_edit = TabReplaceTextEdit(remove_empty_lines=True)

        # ////// TEST AVEC LIGNES VIDES
        text = "line1\n\nline2\n\t\nline3"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "line1\nline2\nline3"

        # ////// TEST SANS LIGNES VIDES
        text = "line1\nline2\nline3"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "line1\nline2\nline3"

    def test_tab_replace_textedit_sanitize_text_preserve_empty_lines(
        self, qt_widget_cleanup
    ):
        """Test de sanitize_text sans suppression des lignes vides."""
        text_edit = TabReplaceTextEdit(remove_empty_lines=False)

        # ////// TEST AVEC LIGNES VIDES PRÉSERVÉES
        text = "line1\n\nline2\n\t\nline3"
        sanitized = text_edit.sanitize_text(text)
        # Les tabs deviennent des newlines par défaut, les lignes vides sont préservées
        assert sanitized == "line1\n\nline2\n\n\nline3"

    def test_tab_replace_textedit_sanitize_text_preserve_whitespace(
        self, qt_widget_cleanup
    ):
        """Test de sanitize_text avec préservation des espaces."""
        text_edit = TabReplaceTextEdit(
            preserve_whitespace=True, remove_empty_lines=True
        )

        # ////// TEST AVEC ESPACES PRÉSERVÉS
        text = "  line1  \n  line2  \n  line3  "
        sanitized = text_edit.sanitize_text(text)
        # Vérifier que les espaces sont préservés
        assert "  line1  " in sanitized
        assert "  line2  " in sanitized
        assert "  line3  " in sanitized

        # ////// TEST SANS ESPACES PRÉSERVÉS
        text_edit.preserve_whitespace = False
        sanitized = text_edit.sanitize_text(text)
        # Les espaces sont préservés mais les lignes vides supprimées
        assert sanitized == "  line1  \n  line2  \n  line3  "

    def test_tab_replace_textedit_sanitize_text_complex(self, qt_widget_cleanup):
        """Test de sanitize_text avec des cas complexes."""
        text_edit = TabReplaceTextEdit(
            tab_replacement="|", remove_empty_lines=True, preserve_whitespace=False
        )

        # ////// TEST CAS COMPLEXE
        text = (
            "  col1\t  col2  \tcol3  \n\n  col4\tcol5\t  col6  \n\t\ncol7\tcol8\tcol9"
        )
        sanitized = text_edit.sanitize_text(text)
        # Les tabs deviennent "|", les lignes vides sont supprimées
        expected = "  col1|  col2  |col3  \n  col4|col5|  col6  \n|\ncol7|col8|col9"
        assert sanitized == expected

    def test_tab_replace_textedit_sanitize_text_mixed_content(self, qt_widget_cleanup):
        """Test de sanitize_text avec du contenu mixte."""
        text_edit = TabReplaceTextEdit(tab_replacement=" -> ", remove_empty_lines=True)

        # ////// TEST CONTENU MIXTE
        text = "header1\theader2\theader3\nvalue1\tvalue2\tvalue3\n\t\t\nfooter1\tfooter2\tfooter3"
        sanitized = text_edit.sanitize_text(text)
        # Les tabs deviennent " -> ", les lignes vides sont supprimées
        expected = "header1 -> header2 -> header3\nvalue1 -> value2 -> value3\n ->  -> \nfooter1 -> footer2 -> footer3"
        assert sanitized == expected

    def test_tab_replace_textedit_sanitize_text_special_characters(
        self, qt_widget_cleanup
    ):
        """Test de sanitize_text avec des caractères spéciaux."""
        text_edit = TabReplaceTextEdit(tab_replacement="\t->\t")

        # ////// TEST CARACTÈRES SPÉCIAUX
        text = "email@domain.com\tuser-name_123\tfile/path\nspecial\tchars\there"
        sanitized = text_edit.sanitize_text(text)
        expected = "email@domain.com\t->\tuser-name_123\t->\tfile/path\nspecial\t->\tchars\t->\there"
        assert sanitized == expected

    def test_tab_replace_textedit_sanitize_text_unicode(self, qt_widget_cleanup):
        """Test de sanitize_text avec des caractères Unicode."""
        text_edit = TabReplaceTextEdit(tab_replacement=" → ")

        # ////// TEST CARACTÈRES UNICODE
        text = "你好\t世界\t测试\némojis\t🚀\t🎉\nunicode\ttext\there"
        sanitized = text_edit.sanitize_text(text)
        expected = "你好 → 世界 → 测试\némojis → 🚀 → 🎉\nunicode → text → here"
        assert sanitized == expected

    def test_tab_replace_textedit_sanitize_text_empty_string(self, qt_widget_cleanup):
        """Test de sanitize_text avec une chaîne vide."""
        text_edit = TabReplaceTextEdit()

        # ////// TEST CHAÎNE VIDE
        sanitized = text_edit.sanitize_text("")
        assert sanitized == ""

        # ////// TEST CHAÎNE AVEC SEULEMENT DES TABS
        sanitized = text_edit.sanitize_text("\t\t\t")
        # Les tabs deviennent des newlines par défaut, puis les lignes vides sont supprimées
        assert sanitized == ""

    def test_tab_replace_textedit_sanitize_text_only_tabs(self, qt_widget_cleanup):
        """Test de sanitize_text avec seulement des tabs."""
        text_edit = TabReplaceTextEdit(tab_replacement="SPACE")

        # ////// TEST SEULEMENT DES TABS
        text = "\t\t\t"
        sanitized = text_edit.sanitize_text(text)
        # Les tabs deviennent "SPACE", puis les lignes vides sont supprimées
        assert sanitized == "SPACESPACESPACE"

    def test_tab_replace_textedit_sanitize_text_multiple_tabs(self, qt_widget_cleanup):
        """Test de sanitize_text avec des tabs multiples consécutifs."""
        text_edit = TabReplaceTextEdit(tab_replacement="|")

        # ////// TEST TABS MULTIPLES
        text = "col1\t\t\tcol2"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "col1|||col2"

    def test_tab_replace_textedit_sanitize_text_edge_cases(self, qt_widget_cleanup):
        """Test de sanitize_text avec des cas limites."""
        text_edit = TabReplaceTextEdit(tab_replacement="TAB")

        # ////// TEST TAB AU DÉBUT
        text = "\tcol1\tcol2"
        sanitized = text_edit.sanitize_text(text)
        # Les tabs deviennent "TAB"
        assert sanitized == "TABcol1TABcol2"

        # ////// TEST TAB À LA FIN
        text = "col1\tcol2\t"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "col1TABcol2TAB"

        # ////// TEST TABS AU DÉBUT ET À LA FIN
        text = "\tcol1\tcol2\t"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "TABcol1TABcol2TAB"

    def test_tab_replace_textedit_text_handling(self, qt_widget_cleanup):
        """Test de la gestion du texte."""
        text_edit = TabReplaceTextEdit()

        # ////// TEST SETPLAINTEXT
        text_edit.setPlainText("test text")
        assert text_edit.toPlainText() == "test text"

        # ////// TEST CLEAR
        text_edit.clear()
        assert text_edit.toPlainText() == ""

        # ////// TEST INSERTPLAINTEXT
        text_edit.insertPlainText("inserted text")
        assert text_edit.toPlainText() == "inserted text"

    def test_tab_replace_textedit_property_type(self, qt_widget_cleanup):
        """Test de la propriété type pour QSS."""
        text_edit = TabReplaceTextEdit()

        # ////// VÉRIFIER QUE LA PROPRIÉTÉ TYPE EST DÉFINIE
        assert text_edit.property("type") == "TabReplaceTextEdit"

    def test_tab_replace_textedit_refresh_style(self, qt_widget_cleanup):
        """Test de la méthode refresh_style."""
        text_edit = TabReplaceTextEdit()

        # ////// LA MÉTHODE NE DOIT PAS LEVER D'EXCEPTION
        try:
            text_edit.refresh_style()
        except Exception as e:
            pytest.fail(f"refresh_style() a levé une exception: {e}")

    def test_tab_replace_textedit_multiple_instances(self, qt_widget_cleanup):
        """Test avec plusieurs instances."""
        text_edit1 = TabReplaceTextEdit(tab_replacement="|")
        text_edit2 = TabReplaceTextEdit(tab_replacement="->")

        # ////// TEST INDÉPENDANCE DES INSTANCES
        text = "col1\tcol2\tcol3"
        sanitized1 = text_edit1.sanitize_text(text)
        sanitized2 = text_edit2.sanitize_text(text)

        assert sanitized1 == "col1|col2|col3"
        assert sanitized2 == "col1->col2->col3"

    def test_tab_replace_textedit_dynamic_property_changes(self, qt_widget_cleanup):
        """Test des changements dynamiques de propriétés."""
        text_edit = TabReplaceTextEdit()
        text = "col1\tcol2\tcol3"

        # ////// TEST CHANGEMENT DYNAMIQUE DE TAB_REPLACEMENT
        text_edit.tab_replacement = "|"
        sanitized1 = text_edit.sanitize_text(text)
        assert sanitized1 == "col1|col2|col3"

        text_edit.tab_replacement = "->"
        sanitized2 = text_edit.sanitize_text(text)
        assert sanitized2 == "col1->col2->col3"

        # ////// TEST CHANGEMENT DYNAMIQUE DE REMOVE_EMPTY_LINES
        text_with_empty = "line1\n\nline2\n\t\nline3"

        text_edit.remove_empty_lines = True
        sanitized3 = text_edit.sanitize_text(text_with_empty)
        assert sanitized3 == "line1\nline2\n->\nline3"

        text_edit.remove_empty_lines = False
        sanitized4 = text_edit.sanitize_text(text_with_empty)
        assert sanitized4 == "line1\n\nline2\n->\nline3"

    def test_tab_replace_textedit_large_text(self, qt_widget_cleanup):
        """Test avec un grand texte."""
        text_edit = TabReplaceTextEdit(tab_replacement="|")

        # ////// CRÉER UN GRAND TEXTE
        lines = []
        for i in range(1000):
            lines.append(f"col1_{i}\tcol2_{i}\tcol3_{i}")
        large_text = "\n".join(lines)

        # ////// SANITISER LE GRAND TEXTE
        sanitized = text_edit.sanitize_text(large_text)

        # ////// VÉRIFIER LE RÉSULTAT
        lines_sanitized = sanitized.split("\n")
        assert len(lines_sanitized) == 1000

        # ////// VÉRIFIER UNE LIGNE SPÉCIFIQUE
        assert "col1_0|col2_0|col3_0" in lines_sanitized[0]
        assert "col1_999|col2_999|col3_999" in lines_sanitized[999]

    def test_tab_replace_textedit_special_replacement_strings(self, qt_widget_cleanup):
        """Test avec des chaînes de remplacement spéciales."""
        text_edit = TabReplaceTextEdit()
        text = "col1\tcol2\tcol3"

        # ////// TEST CHAÎNE VIDE
        text_edit.tab_replacement = ""
        sanitized = text_edit.sanitize_text(text)
        # Chaîne vide = pas de séparation
        assert sanitized == "col1col2col3"

        # ////// TEST CHAÎNE AVEC ESPACES
        text_edit.tab_replacement = "   "
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "col1   col2   col3"

        # ////// TEST CHAÎNE AVEC CARACTÈRES SPÉCIAUX
        text_edit.tab_replacement = "\n\t"
        sanitized = text_edit.sanitize_text(text)
        assert sanitized == "col1\n\tcol2\n\tcol3"



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////

from PySide6.QtCore import (
    QPoint,
    QSize,
    Signal,
    Qt,
    QMimeData,
)
from PySide6.QtGui import (
    QDrag,
    QDragEnterEvent,
    QDragMoveEvent,
    QDropEvent,
    QMouseEvent,
)
from PySide6.QtWidgets import (
    QFrame,
    QHBoxLayout,
    QScrollArea,
    QSizePolicy,
    QVBoxLayout,
    QWidget,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////
from ..label.hover_label import HoverLabel

# ////// TYPE HINTS IMPROVEMENTS FOR PYSIDE6 6.9.1
from typing import Any, Dict, List, Optional, Union

# UTILITY FUNCTIONS
# ///////////////////////////////////////////////////////////////


class DraggableItem(QFrame):
    """
    Widget d'élément draggable pour DraggableList.

    Cet élément peut être déplacé par drag & drop et contient
    toujours un HoverLabel pour une interface cohérente.
    """

    itemClicked = Signal(str)  # Signal émis quand l'élément est cliqué
    itemRemoved = Signal(str)  # Signal émis quand l'élément est supprimé

    def __init__(
        self,
        item_id: str,
        text: str,
        parent: Optional[QWidget] = None,
        icon: Optional[Union[str, Any]] = None,
        compact: bool = False,
        **kwargs: Any,
    ) -> None:
        super().__init__(parent)
        self.setProperty("type", "DraggableItem")

        # Initialisation des attributs
        self.item_id = item_id
        self.text = text
        self.is_dragging = False
        self.drag_start_pos = QPoint()
        self._compact = compact

        # Configuration du widget
        self.setFrameStyle(QFrame.Box)
        self.setLineWidth(1)
        self.setMidLineWidth(0)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)

        # Hauteur selon le mode compact
        if self._compact:
            self.setMinimumHeight(24)
            self.setMaximumHeight(32)
        else:
            self.setMinimumHeight(40)
            self.setMaximumHeight(60)

        # Layout principal
        layout = QHBoxLayout(self)
        if self._compact:
            layout.setContentsMargins(6, 2, 6, 2)  # Marges réduites en mode compact
        else:
            layout.setContentsMargins(8, 4, 8, 4)  # Marges normales
        layout.setSpacing(8)

        # Icône par défaut pour le drag & drop si aucune icône n'est fournie
        if icon is None:
            icon = "https://img.icons8.com/?size=100&id=8329&format=png&color=000000"

        # Widget de contenu (HoverLabel avec icône de suppression)
        icon_size = QSize(16, 16) if self._compact else QSize(20, 20)
        icon_padding = 2 if self._compact else 4

        self.content_widget = HoverLabel(
            text=text,
            icon=icon,  # Icône poubelle pour la suppression
            icon_size=icon_size,
            icon_padding=icon_padding,
            **kwargs,
        )
        self.content_widget.hoverIconClicked.connect(self._on_remove_clicked)

        # Propriété pour la couleur de l'icône
        self._icon_color = "grey"
        # Appliquer la couleur initiale
        self.content_widget.icon_color = self._icon_color

        # Ajout du widget au layout (prend toute la largeur)
        layout.addWidget(self.content_widget)

    def _on_remove_clicked(self) -> None:
        """Gestionnaire de clic sur l'icône de suppression."""
        self.itemRemoved.emit(self.item_id)

    @property
    def icon_color(self) -> str:
        """Obtenir la couleur de l'icône du HoverLabel."""
        return self._icon_color

    @icon_color.setter
    def icon_color(self, value: str) -> None:
        """Définir la couleur de l'icône du HoverLabel."""
        self._icon_color = value
        if self.content_widget:
            self.content_widget.icon_color = value

    @property
    def compact(self) -> bool:
        """Obtenir le mode compact."""
        return self._compact

    @compact.setter
    def compact(self, value: bool) -> None:
        """Définir le mode compact et ajuster la hauteur."""
        self._compact = value
        if self._compact:
            self.setMinimumHeight(24)
            self.setMaximumHeight(32)
        else:
            self.setMinimumHeight(40)
            self.setMaximumHeight(60)
        self.updateGeometry()  # Forcer la mise à jour du layout

    def mousePressEvent(self, event: QMouseEvent) -> None:
        """Début du drag & drop."""
        if event.button() == Qt.LeftButton:
            self.drag_start_pos = event.position().toPoint()
        super().mousePressEvent(event)

    def mouseMoveEvent(self, event: QMouseEvent) -> None:
        """Gestion du mouvement de souris pour le drag & drop."""
        if not (event.buttons() & Qt.LeftButton):
            return

        if not self.is_dragging:
            if (
                event.position().toPoint() - self.drag_start_pos
            ).manhattanLength() < 10:
                return

            self.is_dragging = True
            self.setProperty("dragging", True)
            self.style().unpolish(self)
            self.style().polish(self)

            # Créer le drag
            drag = QDrag(self)
            mime_data = QMimeData()
            mime_data.setText(self.item_id)
            drag.setMimeData(mime_data)

            # Exécuter le drag
            drag.exec(Qt.MoveAction)

            # Nettoyer après le drag
            self.is_dragging = False
            self.setProperty("dragging", False)
            self.style().unpolish(self)
            self.style().polish(self)

    def mouseReleaseEvent(self, event: QMouseEvent) -> None:
        """Fin du drag & drop."""
        self.is_dragging = False
        self.setProperty("dragging", False)
        self.style().unpolish(self)
        self.style().polish(self)
        super().mouseReleaseEvent(event)

    # OVERRIDE FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def sizeHint(self) -> QSize:
        """Taille suggérée du widget basée sur le contenu."""
        # Obtenir la taille suggérée du HoverLabel
        content_size = self.content_widget.sizeHint()

        # Ajouter les marges et le padding du layout
        layout_margins = self.layout().contentsMargins()
        layout_spacing = self.layout().spacing()

        # Calculer la largeur totale
        total_width = (
            content_size.width() + layout_margins.left() + layout_margins.right()
        )

        # Calculer la hauteur totale selon le mode compact
        if self._compact:
            min_height = max(
                24,
                content_size.height() + layout_margins.top() + layout_margins.bottom(),
            )
            max_height = 32
        else:
            min_height = max(
                40,
                content_size.height() + layout_margins.top() + layout_margins.bottom(),
            )
            max_height = 60

        return QSize(total_width, min(min_height, max_height))

    def minimumSizeHint(self) -> QSize:
        """Taille minimale du widget."""
        # Obtenir la taille minimale du HoverLabel
        content_min_size = self.content_widget.minimumSizeHint()

        # Ajouter les marges du layout
        layout_margins = self.layout().contentsMargins()

        # Largeur minimale basée sur le contenu + marges
        min_width = (
            content_min_size.width() + layout_margins.left() + layout_margins.right()
        )

        # Hauteur minimale selon le mode compact
        min_height = 24 if self._compact else 40

        return QSize(min_width, min_height)

    # STYLE FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def refresh_style(self) -> None:
        """Refresh the widget's style (useful after dynamic stylesheet changes)."""
        self.style().unpolish(self)
        self.style().polish(self)
        self.update()


class DraggableList(QWidget):
    """
    Widget de liste d'éléments réorganisables avec drag & drop et suppression.

    Ce widget permet de gérer une liste d'éléments que l'utilisateur peut
    réorganiser par drag & drop et supprimer individuellement.

    Features:
        - Liste d'éléments réorganisables par drag & drop
        - Suppression d'éléments via HoverLabel (icône rouge au survol)
        - Interface cohérente avec HoverLabel pour tous les éléments
        - Signaux pour les événements de réorganisation et suppression
        - Interface fluide et intuitive
        - Personnalisation des apparences
        - Gestion automatique de l'ordre des éléments
        - Icône de suppression intégrée dans HoverLabel

    Example
    -------
    >>> draggable_list = DraggableList(
    ...     items=["Item 1", "Item 2", "Item 3"],
    ...     icon="https://img.icons8.com/?size=100&id=8329&format=png&color=000000"
    ... )
    >>> draggable_list.itemMoved.connect(lambda old_pos, new_pos: print(f"Moved from {old_pos} to {new_pos}"))
    >>> draggable_list.itemRemoved.connect(lambda item_id: print(f"Removed {item_id}"))

    Use cases
    ---------
    - Liste de tâches réorganisables
    - Sélecteur d'options avec ordre personnalisable
    - Interface de gestion de fichiers
    - Configuration d'éléments en ordre de priorité

    Parameters
    ----------
    parent : QWidget, optional
        Le widget parent (default: None).
    items : List[str], optional
        Liste initiale des éléments (default: []).
    allow_drag_drop : bool, optional
        Autoriser le drag & drop pour réorganiser (default: True).
    allow_remove : bool, optional
        Autoriser la suppression d'éléments via HoverLabel (default: True).
    max_height : int, optional
        Hauteur maximale du widget (default: 300).
    min_width : int, optional
        Largeur minimale du widget (default: 200).
    compact : bool, optional
        Afficher les éléments en mode compact (hauteur réduite) (default: False).
    *args, **kwargs :
        Arguments supplémentaires passés aux widgets d'éléments.

    Properties
    ----------
    items : List[str]
        Obtenir ou définir la liste des éléments.
    item_count : int
        Nombre d'éléments dans la liste (lecture seule).
    allow_drag_drop : bool
        Obtenir ou définir l'autorisation de drag & drop.
    allow_remove : bool
        Obtenir ou définir l'autorisation de suppression.
    icon_color : str
        Obtenir ou définir la couleur de l'icône des éléments.
    compact : bool
        Obtenir ou définir le mode compact.
    min_width : int
        Obtenir ou définir la largeur minimale du widget.

    Signals
    -------
    itemMoved(str, int, int)
        Émis quand un élément est déplacé (item_id, old_position, new_position).
    itemRemoved(str, int)
        Émis quand un élément est supprimé (item_id, position).
    itemAdded(str, int)
        Émis quand un élément est ajouté (item_id, position).
    itemClicked(str)
        Émis quand un élément est cliqué (item_id).
    orderChanged(List[str])
        Émis quand l'ordre des éléments change (nouvelle liste ordonnée).
    """

    itemMoved = Signal(str, int, int)  # item_id, old_position, new_position
    itemRemoved = Signal(str, int)  # item_id, position
    itemAdded = Signal(str, int)  # item_id, position
    itemClicked = Signal(str)  # item_id
    orderChanged = Signal(list)  # nouvelle liste ordonnée

    def __init__(
        self,
        parent: Optional[QWidget] = None,
        items: Optional[List[str]] = None,
        allow_drag_drop: bool = True,
        allow_remove: bool = True,
        max_height: int = 300,
        min_width: int = 150,
        compact: bool = False,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        super().__init__(parent)
        self.setProperty("type", "DraggableList")

        # Initialisation des attributs
        self._items: List[str] = items or []
        self._allow_drag_drop: bool = allow_drag_drop
        self._allow_remove: bool = allow_remove
        self._max_height: int = max_height
        self._min_width: int = min_width
        self._compact: bool = compact
        self._item_widgets: Dict[str, DraggableItem] = {}  # item_id -> DraggableItem
        self._kwargs = kwargs
        self._icon_color = "grey"  # Couleur par défaut de l'icône

        # Configuration du widget
        self.setAcceptDrops(True)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.setMinimumWidth(min_width)
        self.setMaximumHeight(max_height)

        # Layout principal
        layout = QVBoxLayout(self)
        layout.setContentsMargins(8, 8, 8, 8)
        layout.setSpacing(4)

        # Zone de scroll
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scroll_area.setFrameStyle(QFrame.NoFrame)

        # Widget conteneur pour les éléments
        self.container_widget = QWidget()
        self.container_layout = QVBoxLayout(self.container_widget)
        self.container_layout.setContentsMargins(0, 0, 0, 0)
        self.container_layout.setSpacing(4)
        self.container_layout.addStretch()  # Espace flexible à la fin

        self.scroll_area.setWidget(self.container_widget)
        layout.addWidget(self.scroll_area)

        # Initialiser les éléments
        self._create_items()

    # PROPERTIES
    # ///////////////////////////////////////////////////////////////

    @property
    def items(self) -> List[str]:
        """Obtenir la liste des éléments."""
        return self._items.copy()

    @items.setter
    def items(self, value: List[str]) -> None:
        """Définir la liste des éléments."""
        self._items = value.copy()
        self._create_items()

    @property
    def item_count(self) -> int:
        """Nombre d'éléments dans la liste."""
        return len(self._items)

    @property
    def allow_drag_drop(self) -> bool:
        """Obtenir l'autorisation de drag & drop."""
        return self._allow_drag_drop

    @allow_drag_drop.setter
    def allow_drag_drop(self, value: bool) -> None:
        """Définir l'autorisation de drag & drop."""
        self._allow_drag_drop = value

    @property
    def allow_remove(self) -> bool:
        """Obtenir l'autorisation de suppression."""
        return self._allow_remove

    @allow_remove.setter
    def allow_remove(self, value: bool) -> None:
        """Définir l'autorisation de suppression."""
        self._allow_remove = value
        for widget in self._item_widgets.values():
            widget.content_widget.icon_enabled = value

    @property
    def icon_color(self) -> str:
        """Obtenir la couleur de l'icône des éléments."""
        return self._icon_color

    @icon_color.setter
    def icon_color(self, value: str) -> None:
        """Définir la couleur de l'icône de tous les éléments."""
        self._icon_color = value
        for widget in self._item_widgets.values():
            widget.icon_color = value

    @property
    def compact(self) -> bool:
        """Obtenir le mode compact."""
        return self._compact

    @compact.setter
    def compact(self, value: bool) -> None:
        """Définir le mode compact et mettre à jour tous les éléments."""
        self._compact = value
        for widget in self._item_widgets.values():
            widget.compact = value

    @property
    def min_width(self) -> int:
        """Obtenir la largeur minimale du widget."""
        return self._min_width

    @min_width.setter
    def min_width(self, value: int) -> None:
        """Définir la largeur minimale du widget."""
        self._min_width = value
        self.updateGeometry()  # Forcer la mise à jour du layout

    # ITEM MANAGEMENT FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def add_item(self, item_id: str, text: Optional[str] = None) -> None:
        """
        Ajouter un élément à la liste.

        Parameters
        ----------
        item_id : str
            Identifiant unique de l'élément.
        text : str, optional
            Texte à afficher (utilise item_id si None).
        """
        if item_id in self._items:
            return  # Élément déjà présent

        text = text or item_id
        self._items.append(item_id)

        # Créer le widget
        item_widget = DraggableItem(
            item_id=item_id, text=text, compact=self._compact, **self._kwargs
        )

        # Connecter les signaux
        item_widget.itemRemoved.connect(self._on_item_removed)

        # Masquer l'icône de suppression si nécessaire
        if not self._allow_remove:
            item_widget.content_widget.icon_enabled = False

        # Ajouter au layout (avant le stretch)
        self.container_layout.insertWidget(len(self._items) - 1, item_widget)
        self._item_widgets[item_id] = item_widget

        # Émettre le signal
        self.itemAdded.emit(item_id, len(self._items) - 1)
        self.orderChanged.emit(self._items.copy())

    def remove_item(self, item_id: str) -> bool:
        """
        Supprimer un élément de la liste.

        Parameters
        ----------
        item_id : str
            Identifiant de l'élément à supprimer.

        Returns
        -------
        bool
            True si l'élément a été supprimé, False sinon.
        """
        if item_id not in self._items:
            return False

        # Supprimer de la liste
        position = self._items.index(item_id)
        self._items.remove(item_id)

        # Supprimer le widget
        if item_id in self._item_widgets:
            widget = self._item_widgets[item_id]
            self.container_layout.removeWidget(widget)
            widget.deleteLater()
            del self._item_widgets[item_id]

        # Émettre les signaux
        self.itemRemoved.emit(item_id, position)
        self.orderChanged.emit(self._items.copy())

        return True

    def clear_items(self) -> None:
        """Supprimer tous les éléments de la liste."""
        # Nettoyer les widgets
        for widget in self._item_widgets.values():
            self.container_layout.removeWidget(widget)
            widget.deleteLater()
        self._item_widgets.clear()

        # Vider la liste
        self._items.clear()

        # Émettre le signal
        self.orderChanged.emit([])

    def move_item(self, item_id: str, new_position: int) -> bool:
        """
        Déplacer un élément à une nouvelle position.

        Parameters
        ----------
        item_id : str
            Identifiant de l'élément à déplacer.
        new_position : int
            Nouvelle position (0-based).

        Returns
        -------
        bool
            True si l'élément a été déplacé, False sinon.
        """
        if item_id not in self._items:
            return False

        old_position = self._items.index(item_id)
        if old_position == new_position:
            return True

        # Déplacer dans la liste
        self._items.pop(old_position)
        self._items.insert(new_position, item_id)

        # Déplacer le widget
        if item_id in self._item_widgets:
            widget = self._item_widgets[item_id]
            self.container_layout.removeWidget(widget)
            self.container_layout.insertWidget(new_position, widget)

        # Émettre les signaux
        self.itemMoved.emit(item_id, old_position, new_position)
        self.orderChanged.emit(self._items.copy())

        return True

    def get_item_position(self, item_id: str) -> int:
        """
        Obtenir la position d'un élément.

        Parameters
        ----------
        item_id : str
            Identifiant de l'élément.

        Returns
        -------
        int
            Position de l'élément (-1 si non trouvé).
        """
        try:
            return self._items.index(item_id)
        except ValueError:
            return -1

    def _create_items(self) -> None:
        """Créer les widgets pour tous les éléments."""
        # Nettoyer les widgets existants
        for widget in self._item_widgets.values():
            self.container_layout.removeWidget(widget)
            widget.deleteLater()
        self._item_widgets.clear()

        # Créer les nouveaux widgets
        for i, item_id in enumerate(self._items):
            item_widget = DraggableItem(
                item_id=item_id, text=item_id, compact=self._compact, **self._kwargs
            )

            # Connecter les signaux
            item_widget.itemRemoved.connect(self._on_item_removed)

            # Masquer l'icône de suppression si nécessaire
            if not self._allow_remove:
                item_widget.content_widget.icon_enabled = False

            # Ajouter au layout
            self.container_layout.insertWidget(i, item_widget)
            self._item_widgets[item_id] = item_widget

    def _on_item_removed(self, item_id: str) -> None:
        """Gestionnaire de suppression d'un élément."""
        self.remove_item(item_id)

    # EVENT FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def dragEnterEvent(self, event: QDragEnterEvent) -> None:
        """Gestionnaire d'entrée de drag."""
        if self._allow_drag_drop and event.mimeData().hasText():
            event.acceptProposedAction()

    def dragMoveEvent(self, event: QDragMoveEvent) -> None:
        """Gestionnaire de mouvement de drag."""
        if self._allow_drag_drop and event.mimeData().hasText():
            event.acceptProposedAction()

    def dropEvent(self, event: QDropEvent) -> None:
        """Gestionnaire de drop."""
        if not self._allow_drag_drop:
            return

        item_id = event.mimeData().text()
        if item_id not in self._items:
            return

        # Calculer la nouvelle position
        drop_pos = event.position().toPoint()
        new_position = self._calculate_drop_position(drop_pos)

        # Déplacer l'élément
        self.move_item(item_id, new_position)

        event.acceptProposedAction()

    # UI FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def _calculate_drop_position(self, drop_pos: QPoint) -> int:
        """Calculer la position de drop basée sur les coordonnées."""
        # Convertir les coordonnées globales en coordonnées locales du container
        local_pos = self.container_widget.mapFrom(self, drop_pos)

        # Trouver la position dans le layout
        for i in range(self.container_layout.count() - 1):  # -1 pour exclure le stretch
            item = self.container_layout.itemAt(i)
            if item.widget():
                widget_rect = item.widget().geometry()
                if local_pos.y() < widget_rect.center().y():
                    return i

        return len(self._items) - 1

    # OVERRIDE FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def sizeHint(self) -> QSize:
        """Taille suggérée du widget basée sur le contenu."""
        # Calculer la largeur maximale des éléments
        max_item_width = 0

        if self._item_widgets:
            # Obtenir la largeur maximale des éléments existants
            item_widths = [
                widget.sizeHint().width() for widget in self._item_widgets.values()
            ]
            max_item_width = max(item_widths) if item_widths else 0

        # Utiliser la largeur minimale seulement si nécessaire
        if max_item_width < self._min_width:
            max_item_width = self._min_width

        # Ajouter les marges du widget principal
        margins = self.contentsMargins()
        total_width = max_item_width + margins.left() + margins.right()

        # Calculer la hauteur basée sur le nombre d'éléments
        item_height = 50  # Hauteur approximative d'un élément
        spacing = 4  # Espacement entre éléments
        total_items_height = len(self._item_widgets) * (item_height + spacing)

        # Ajouter les marges et limiter à la hauteur maximale
        total_height = min(
            total_items_height + margins.top() + margins.bottom(), self._max_height
        )

        return QSize(total_width, max(200, total_height))

    def minimumSizeHint(self) -> QSize:
        """Taille minimale du widget."""
        # Largeur minimale basée sur les éléments ou la largeur minimale configurée
        min_width = 0

        if self._item_widgets:
            # Obtenir la largeur minimale des éléments existants
            item_min_widths = [
                widget.minimumSizeHint().width()
                for widget in self._item_widgets.values()
            ]
            min_width = max(item_min_widths) if item_min_widths else 0

        # Utiliser la largeur minimale seulement si nécessaire
        if min_width < self._min_width:
            min_width = self._min_width

        # Ajouter les marges
        margins = self.contentsMargins()
        total_width = min_width + margins.left() + margins.right()

        # Hauteur minimale basée sur au moins un élément
        item_min_height = 40  # Hauteur minimale d'un élément
        spacing = 4  # Espacement
        min_height = item_min_height + spacing + margins.top() + margins.bottom()

        return QSize(total_width, min_height)

    # STYLE FUNCTIONS
    # ///////////////////////////////////////////////////////////////

    def refresh_style(self) -> None:
        """Refresh the widget's style (useful after dynamic stylesheet changes)."""
        self.style().unpolish(self)
        self.style().polish(self)
        self.update()



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////
"""
Tests unitaires pour le widget OptionSelector.
"""

import pytest
from PySide6.QtCore import Qt
from ezqt_widgets.misc.option_selector import OptionSelector


pytestmark = pytest.mark.unit


class TestOptionSelector:
    """Test cases for OptionSelector widget."""

    def test_option_selector_creation_default(self, qt_widget_cleanup):
        """Test OptionSelector creation with default parameters."""
        items = ["Option 1", "Option 2", "Option 3"]
        selector = OptionSelector(items)

        assert selector.value == "Option 1"  # Default selection
        assert selector.value_id == 0
        assert selector.options == items
        assert selector.default_id == 0
        assert selector.orientation == "horizontal"
        assert selector.animation_duration == 300

    def test_option_selector_creation_custom(self, qt_widget_cleanup):
        """Test OptionSelector creation with custom parameters."""
        items = ["A", "B", "C", "D"]
        selector = OptionSelector(
            items=items, default_id=2, orientation="vertical", animation_duration=500
        )

        assert selector.value == "C"  # Index 2
        assert selector.value_id == 2
        assert selector.options == items
        assert selector.default_id == 2
        assert selector.orientation == "vertical"
        assert selector.animation_duration == 500

    def test_option_selector_add_option(self, qt_widget_cleanup):
        """Test adding options to the selector."""
        items = ["Option 1", "Option 2"]
        selector = OptionSelector(items)

        # Add a new option
        selector.add_option(2, "Option 3")

        assert len(selector.options) == 3
        assert "Option 3" in selector.options

    def test_option_selector_set_value_id(self, qt_widget_cleanup):
        """Test setting value by ID."""
        items = ["Option 1", "Option 2", "Option 3"]
        selector = OptionSelector(items)

        selector.value_id = 2
        assert selector.value == "Option 3"
        assert selector.value_id == 2

    def test_option_selector_set_value(self, qt_widget_cleanup):
        """Test setting value by text."""
        items = ["Option 1", "Option 2", "Option 3"]
        selector = OptionSelector(items)

        selector.value = "Option 2"
        assert selector.value == "Option 2"
        assert selector.value_id == 1

    def test_option_selector_signals(self, qt_widget_cleanup):
        """Test option selector signals."""
        items = ["Option 1", "Option 2"]
        selector = OptionSelector(items)

        clicked_called = False
        value_changed_called = False
        value_id_changed_called = False

        def on_clicked():
            nonlocal clicked_called
            clicked_called = True

        def on_value_changed(value):
            nonlocal value_changed_called
            value_changed_called = True

        def on_value_id_changed(value_id):
            nonlocal value_id_changed_called
            value_id_changed_called = True

        selector.clicked.connect(on_clicked)
        selector.valueChanged.connect(on_value_changed)
        selector.valueIdChanged.connect(on_value_id_changed)

        # Change value
        selector.value = "Option 2"
        assert value_changed_called
        assert value_id_changed_called

    def test_option_selector_properties(self, qt_widget_cleanup):
        """Test option selector properties."""
        items = ["Option 1", "Option 2"]
        selector = OptionSelector(items)

        # Test default_id setter
        selector.default_id = 1
        assert selector.default_id == 1

        # Test orientation setter
        selector.orientation = "vertical"
        assert selector.orientation == "vertical"

        # Test min_width setter
        selector.min_width = 200
        assert selector.min_width == 200

        # Test min_height setter
        selector.min_height = 100
        assert selector.min_height == 100

        # Test animation_duration setter
        selector.animation_duration = 400
        assert selector.animation_duration == 400

    def test_option_selector_toggle_selection(self, qt_widget_cleanup):
        """Test toggling selection."""
        items = ["Option 1", "Option 2", "Option 3"]
        selector = OptionSelector(items)

        # Toggle to option 2
        selector.toggle_selection(1)
        assert selector.value_id == 1
        assert selector.value == "Option 2"

    def test_option_selector_selected_option_property(self, qt_widget_cleanup):
        """Test getting the selected option widget."""
        items = ["Option 1", "Option 2"]
        selector = OptionSelector(items)

        selected_option = selector.selected_option
        assert selected_option is not None
        assert selected_option.text() == "Option 1"

    @pytest.mark.skip(reason="TypeError: 'str' object is not callable")
    def test_option_selector_size_hints(self, qt_widget_cleanup):
        """Test size hint methods."""
        items = ["Option 1", "Option 2"]
        selector = OptionSelector(items)

        size_hint = selector.sizeHint()
        assert size_hint.width() > 0
        assert size_hint.height() > 0

        min_size_hint = selector.minimumSizeHint()
        assert min_size_hint.width() > 0
        assert min_size_hint.height() > 0
